import unittest

from tala.ddd.parser import Parser, ParseError
from tala.model.action import Action
from tala.model.domain import Domain
from tala.model.goal import PerformGoal, ResolveGoal, HandleGoal
from tala.model.individual import Individual, Yes, No
from tala.model.lambda_abstraction import LambdaAbstractedGoalProposition
from tala.model.lambda_abstraction import LambdaAbstractedPredicateProposition
from tala.model.speaker import Speaker
from tala.model.set import Set
from tala.model.move import ICMMove, ICMMoveWithStringContent, ICMMoveWithSemanticContent, IssueICMMove, Move, ReportMove, GreetMove, MuteMove, PrereportMove
from tala.model.ontology import Ontology
from tala.model.plan_item import AssumePlanItem, AssumeSharedPlanItem, AssumeIssuePlanItem, RespondPlanItem, EmitIcmPlanItem, BindPlanItem, ConsultDBPlanItem, JumpToPlanItem, IfThenElse
from tala.model.polarity import Polarity
from tala.model.predicate import Predicate
from tala.model.proposition import ResolvednessProposition, RejectedPropositions, ServiceResultProposition, GoalProposition, ServiceActionStartedProposition, ServiceActionTerminatedProposition, PropositionSet, PredicateProposition
from tala.model.question import WhQuestion
from tala.model.question_raising_plan_item import QuestionRaisingPlanItem, FindoutPlanItem, RaisePlanItem
from tala.model.sort import CustomSort, RealSort, IntegerSort, StringSort, BooleanSort
from tala.model.service_action_outcome import SuccessfulServiceAction, FailedServiceAction
from tala.testing.move_factory import MoveFactoryWithPredefinedBoilerplate


class ParserTests(unittest.TestCase):
    def setUp(self):
        self._ddd_name = "mockup_ddd"
        self._create_ontology()
        self._create_domain()
        self._create_semantic_objects()
        self.parser = Parser(self._ddd_name, self.ontology, self.domain_name)
        self._move_factory = MoveFactoryWithPredefinedBoilerplate(self.ontology.name)

    def _create_ontology(self):
        self.ontology_name = "mockup_ontology"
        sorts = set([CustomSort(self.ontology_name, "city")])
        predicates = set([
            self._create_predicate("dest_city", CustomSort(self.ontology_name, "city")),
            self._create_predicate("dept_city", CustomSort(self.ontology_name, "city")),
            self._create_predicate("available_dest_city", CustomSort(self.ontology_name, "city")),
            self._create_predicate("price", RealSort()),
            self._create_predicate("number_of_passengers", IntegerSort()),
            self._create_predicate("number_to_call", StringSort()),
            self._create_predicate("need_visa", BooleanSort()),
        ])
        individuals = {
            "paris": CustomSort(self.ontology_name, "city"),
            "london": CustomSort(self.ontology_name, "city")
        }
        actions = set(["top", "buy"])
        self.ontology = Ontology(self.ontology_name, sorts, predicates, individuals, actions)

        self.empty_ontology = Ontology("empty_ontology", {}, {}, {}, set([]))

    def _create_predicate(self, *args, **kwargs):
        return Predicate(self.ontology_name, *args, **kwargs)

    def _create_domain(self):
        self.domain_name = "mockup_domain"
        self.domain = Domain(self._ddd_name, self.domain_name, self.ontology, plans={})

    def _create_semantic_objects(self):
        self.predicate_dest_city = self.ontology.get_predicate("dest_city")
        self.predicate_dept_city = self.ontology.get_predicate("dept_city")
        self.predicate_price = self.ontology.get_predicate("price")
        self.predicate_number_to_call = self.ontology.get_predicate("number_to_call")
        self.predicate_need_visa = self.ontology.get_predicate("need_visa")
        self.individual_paris = self.ontology.create_individual("paris")
        self.individual_not_paris = self.ontology.create_negative_individual("paris")
        self.individual_london = self.ontology.create_individual("london")
        self.real_individual = self.ontology.create_individual(1234.0)
        self.integer_individual = self.ontology.create_individual(1234)

        self.proposition_dest_city_paris = PredicateProposition(self.predicate_dest_city, self.individual_paris)
        self.proposition_not_dest_city_paris = PredicateProposition(
            self.predicate_dest_city, self.individual_paris, Polarity.NEG
        )
        self.proposition_dest_city_london = PredicateProposition(self.predicate_dest_city, self.individual_london)
        self.price_proposition = PredicateProposition(self.predicate_price, self.real_individual)
        self.proposition_need_visa = PredicateProposition(self.predicate_need_visa)
        self.proposition_not_need_visa = PredicateProposition(self.predicate_need_visa, polarity=Polarity.NEG)

        self.lambda_abstracted_dest_city_prop = LambdaAbstractedPredicateProposition(
            self.predicate_dest_city, self.ontology_name
        )
        self.lambda_abstracted_price_prop = LambdaAbstractedPredicateProposition(
            self.predicate_price, self.ontology_name
        )

        self.dest_city_question = WhQuestion(self.lambda_abstracted_dest_city_prop)
        self.price_question = WhQuestion(self.lambda_abstracted_price_prop)

        self.buy_action = self.ontology.create_action("buy")
        self.top_action = self.ontology.create_action("top")

    def test_understanding_proposition(self):
        und_string = "und(USR, dest_city(paris))"
        und = self.parser.parse(und_string)
        self.assertTrue(und.is_understanding_proposition())
        self.assertEquals(Speaker.USR, und.get_speaker())

    def test_neg_understanding_proposition(self):
        und_string = "~und(USR, dest_city(paris))"
        und = self.parser.parse(und_string)
        self.assertTrue(und.is_understanding_proposition())

    def test_understanding_question(self):
        und_string = "?und(USR, dest_city(paris))"
        und = self.parser.parse(und_string)
        self.assertTrue(und.is_understanding_question())

    def test_understanding_proposition_with_speaker_none(self):
        und_string = "und(None, dest_city(paris))"
        und = self.parser.parse(und_string)
        self.assertTrue(und.is_understanding_proposition())
        self.assertEquals(None, und.get_speaker())

    def test_create_yes_no_question(self):
        question = self.parser.parse("?dest_city(paris)")  # noqa: F841

    def test_create_nullary_yes_no_question_I(self):
        question = self.parser.parse("?need_visa()")  # noqa: F841

    def test_create_yes(self):
        yes = self.parser.parse("yes")
        self.assertEquals(Yes(), yes)

    def test_create_no(self):
        no = self.parser.parse("no")
        self.assertEquals(No(), no)

    def test_create_yes_answer_move(self):
        yes_move = self.parser.parse("answer(yes)")
        self.assertEquals(Yes(), yes_move.get_content())

    def test_create_no_answer_move(self):
        no_move = self.parser.parse("answer(no)")
        self.assertEquals(No(), no_move.get_content())

    def test_ShortAnswer(self):
        answer_move_from_string = self.parser.parse("answer(paris)")
        answer = self.individual_paris
        answer_move = self._move_factory.createAnswerMove(answer)
        self.assertEquals(answer_move, answer_move_from_string)

    def test_PropositionalAnswerMove(self):
        answer_move_from_string = self.parser.parse("answer(dest_city(paris))")
        individual = self.individual_paris
        answer = PredicateProposition(self.predicate_dest_city, individual)
        answer_move = self._move_factory.createAnswerMove(answer)
        self.assertEquals(answer_move, answer_move_from_string)

    def test_integer_short_answer(self):
        move = self.parser.parse("answer(1234)")
        expected_move = self._move_factory.createAnswerMove(self.integer_individual)
        self.assertEquals(expected_move, move)

    def test_integer_short_answer_with_ontology_without_real_sort(self):
        sorts = set([])
        predicates = set([self._create_predicate("number_of_passengers", IntegerSort())])
        individuals = {}
        actions = set([])
        ontology_without_real_sort = Ontology("ontology_without_real_sort", sorts, predicates, individuals, actions)
        move = Parser(self._ddd_name, ontology_without_real_sort, self.domain_name).parse("answer(1234)")
        self.assertEqual(move.get_type(), Move.ANSWER)
        self.assertTrue(move.get_content().is_individual())
        self.assertEqual(move.get_content().getSort().get_name(), "integer")

    def test_float_short_answer(self):
        move = self.parser.parse("answer(1234.0)")
        expected_move = self._move_factory.createAnswerMove(self.real_individual)
        self.assertEquals(expected_move, move)

    def test_float_answer(self):
        move = self.parser.parse("answer(price(1234.0))")
        expected_move = self._move_factory.createAnswerMove(self.price_proposition)
        self.assertEquals(expected_move, move)

    def test_propositional_usr_answer_move_w_score(self):
        self._when_parse(
            "Move(answer(dest_city(paris)), understanding_confidence=0.45, speaker=USR, ddd_name='mockup_ddd')"
        )
        self._then_result_is(
            MoveFactoryWithPredefinedBoilerplate(
                self.ontology.name, understanding_confidence=0.45, speaker=Speaker.USR, ddd_name="mockup_ddd"
            ).createAnswerMove(PredicateProposition(self.predicate_dest_city, self.individual_paris))
        )

    def _when_parse(self, string):
        self._result = self.parser.parse(string)

    def _then_result_is(self, expected_result):
        self.assertEquals(expected_result, self._result)

    def test_move_with_utterance(self):
        self._when_parse(
            "Move(answer(dest_city(paris)), understanding_confidence=0.45, speaker=USR, "
            "utterance='paris', ddd_name='mockup_ddd')"
        )
        self._then_result_is(
            MoveFactoryWithPredefinedBoilerplate(
                self.ontology.name,
                ddd_name="mockup_ddd",
                understanding_confidence=0.45,
                speaker=Speaker.USR,
                utterance="paris"
            ).createAnswerMove(PredicateProposition(self.predicate_dest_city, self.individual_paris))
        )

    def test_undecorated_move_with_Move_classname(self):
        string = "Move(ask(?set([goal(perform(top)), goal(perform(buy))])))"
        expected_move = self.parser.parse("ask(?set([goal(perform(top)), goal(perform(buy))]))")
        self.assertEquals(expected_move, self.parser.parse(string))

    def test_Individual(self):
        individual_from_string = self.parser.parse("paris")
        self.assertEquals(self.individual_paris, individual_from_string)

    def test_NegativeIndividual(self):
        individual_from_string = self.parser.parse("~paris")
        self.assertEquals(self.individual_not_paris, individual_from_string)

    def test_ask_move(self):
        move_from_string = self.parser.parse("ask(?X.dest_city(X))")
        question = self.parser.parse("?X.dest_city(X)")
        move = self._move_factory.create_ask_move(question)
        self.assertEquals(move, move_from_string)

    def test_ask_move_with_speaker_score(self):
        score = 1.0
        speaker = Speaker.SYS
        move_from_string = self.parser.parse(
            "Move(ask(?X.dest_city(X)), speaker=%s, understanding_confidence=%s, ddd_name='mockup_ddd')" %
            (speaker, score)
        )
        question = self.parser.parse("?X.dest_city(X)")
        move = MoveFactoryWithPredefinedBoilerplate(
            self.ontology.name, speaker=speaker, understanding_confidence=score, ddd_name="mockup_ddd"
        ).create_ask_move(question)
        self.assertEquals(move, move_from_string)

    def test_create_request_move(self):
        request = self.parser.parse("request(buy)")
        self.assertEquals(Move.REQUEST, request.get_type())

    def test_create_request_move_w_score_and_usr(self):
        score = 0.55
        speaker = Speaker.USR
        expected_move = self._move_factory.createMove(
            Move.REQUEST,
            content=Action("buy", self.ontology.get_name()),
            speaker=speaker,
            understanding_confidence=score,
            ddd_name="mockup_ddd"
        )
        request = self.parser.parse(
            "Move(request(buy), speaker=%s, understanding_confidence=%s, ddd_name='mockup_ddd')" % (speaker, score)
        )
        self.assertEquals(expected_move, request)

    def test_create_set_with_single_element(self):
        set_of_moves = self.parser.parse("{request(top)}")
        expected_result = Set(Move)
        expected_result.add(self.parser.parse("request(top)"))
        self.assertEquals(expected_result, set_of_moves)

    def test_create_empty_set(self):
        empty_set = self.parser.parse("{}")
        expected_result = Set()
        self.assertEquals(expected_result, empty_set)

    def test_lambda_abstracted_predicate_proposition(self):
        object = self.parser.parse("X.dest_city(X)")
        expected_object = self.lambda_abstracted_dest_city_prop
        self.assertEquals(expected_object, object)

    def test_lambda_abstracted_goal_proposition(self):
        object = self.parser.parse("X.goal(X)")
        expected_object = LambdaAbstractedGoalProposition()
        self.assertEquals(expected_object, object)

    def test_action(self):
        object = self.parser.parse("buy")
        expected_object = self.buy_action
        self.assertEquals(expected_object, object)

    def test_unary_predicate_proposition(self):
        proposition_from_string = self.parser.parse("dest_city(paris)")
        self.assertEquals(self.proposition_dest_city_paris, proposition_from_string)

    def test_nullary_predicate_proposition(self):
        proposition_from_string = self.parser.parse("need_visa()")
        self.assertEquals(self.proposition_need_visa, proposition_from_string)

    def test_negative_unary_predicate_proposition(self):
        proposition_from_string = self.parser.parse("~dest_city(paris)")
        self.assertEquals(self.proposition_not_dest_city_paris, proposition_from_string)

    def test_negative_nullary_predicate_proposition(self):
        proposition_from_string = self.parser.parse("~need_visa()")
        self.assertEquals(self.proposition_not_need_visa, proposition_from_string)

    def test_positive_perform_proposition(self):
        proposition = self.parser.parse("goal(perform(buy))")
        self.assertTrue(proposition.is_positive())
        self.assertEquals(PerformGoal(self.buy_action), proposition.get_goal())

    def test_negative_action_proposition(self):
        proposition = self.parser.parse("~goal(perform(buy))")
        self.assertFalse(proposition.is_positive())
        self.assertEquals(PerformGoal(self.buy_action), proposition.get_goal())

    def test_illegal_perform_goal(self):
        with self.assertRaises(ParseError):
            self.parser.parse("perform(sdfkjsd)")

    def test_issue_proposition(self):
        object = self.parser.parse("goal(resolve_user(?X.dest_city(X)))")
        expected_object = GoalProposition(ResolveGoal(self.dest_city_question, Speaker.USR))
        self.assertEquals(expected_object, object)

    def test_resolve_goal_containing_non_question_yields_exception(self):
        with self.assertRaises(ParseError):
            self.parser.parse("resolve(buy)")

    def test_resolvedness_proposition(self):
        object = self.parser.parse("resolved(?X.dest_city(X))")
        expected_object = ResolvednessProposition(self.dest_city_question)
        self.assertEquals(expected_object, object)

    def test_positive_preconfirmation(self):
        preconfirmation = self.parser.parse("preconfirmed(MakeReservation, [])")
        self.assertTrue(preconfirmation.is_positive())
        self.assertEquals("MakeReservation", preconfirmation.get_service_action())
        self.assertEquals([], preconfirmation.get_arguments())

    def test_negative_preconfirmation(self):
        preconfirmation = self.parser.parse("~preconfirmed(MakeReservation, [])")
        self.assertFalse(preconfirmation.is_positive())
        self.assertEquals("MakeReservation", preconfirmation.get_service_action())
        self.assertEquals([], preconfirmation.get_arguments())

    def test_preconfirmation_w_single_param(self):
        preconfirmation = self.parser.parse("~preconfirmed(MakeReservation, [dest_city(paris)])")
        self.assertEquals(1, len(preconfirmation.get_arguments()))
        self.assertEquals("dest_city(paris)", unicode(preconfirmation.get_arguments()[0]))

    def test_preconfirmation_q_w_single_param(self):
        preconfirmation = self.parser.parse("?preconfirmed(MakeReservation, [dest_city(paris)])")  # noqa: F841

    def test_preconfirmation_q_w_multi_param(self):
        service_action = "MakeReservation"
        prop_1 = "dest_city(paris)"
        prop_2 = "dest_city(london)"
        preconfirmed_string = "?preconfirmed(%s, [%s, %s])" % (service_action, prop_1, prop_2)
        preconfirmation = self.parser.parse(preconfirmed_string)  # noqa: F841

    def test_rejected_proposition(self):
        object = self.parser.parse("rejected(set([dest_city(paris)]))")
        expected_object = RejectedPropositions(PropositionSet([self.proposition_dest_city_paris]))
        self.assertEquals(expected_object, object)

    def test_rejected_proposition_with_reason(self):
        object = self.parser.parse("rejected(set([dest_city(paris)]), some_reason)")
        expected_object = RejectedPropositions(PropositionSet([self.proposition_dest_city_paris]), reason="some_reason")
        self.assertEquals(expected_object, object)

    def test_create_prereport(self):
        confirmation = self.parser.parse("prereported(MakeReservation, [])")
        self.assertEquals("MakeReservation", confirmation.get_service_action())
        self.assertEquals([], confirmation.get_arguments())

    def test_create_prereport_w_single_param(self):
        prereport = self.parser.parse("prereported(MakeReservation, [dest_city(paris)])")
        self.assertEquals(1, len(prereport.get_arguments()))
        self.assertEquals("dest_city(paris)", unicode(prereport.get_arguments()[0]))

    def test_create_prereport_w_multi_param(self):
        service_action = "MakeReservation"
        prop_1 = "dest_city(paris)"
        prop_2 = "dest_city(london)"
        prereport_string = "?prereported(%s, [%s, %s])" % (service_action, prop_1, prop_2)
        prereport = self.parser.parse(prereport_string)  # noqa: F841

    def test_report_move_successful_with_parameters(self):
        object = self.parser.parse(
            "report(ServiceResultProposition("
            "MakeReservation, [dest_city(paris), dest_city(london)], SuccessfulServiceAction()))"
        )
        expected_object = ReportMove(
            ServiceResultProposition(
                self.ontology_name, "MakeReservation",
                [self.proposition_dest_city_paris, self.proposition_dest_city_london], SuccessfulServiceAction()
            )
        )
        self.assertEquals(expected_object, object)

    def test_report_move_successful(self):
        object = self.parser.parse("report(ServiceResultProposition(MakeReservation, [], SuccessfulServiceAction()))")
        expected_object = ReportMove(
            ServiceResultProposition(self.ontology_name, "MakeReservation", [], SuccessfulServiceAction())
        )
        self.assertEquals(expected_object, object)

    def test_report_move_failed(self):
        object = self.parser.parse(
            "report(ServiceResultProposition(MakeReservation, [], FailedServiceAction(no_itinerary_found)))"
        )
        expected_object = ReportMove(
            ServiceResultProposition(
                self.ontology_name, "MakeReservation", [], FailedServiceAction("no_itinerary_found")
            )
        )
        self.assertEquals(expected_object, object)

    def test_prereport_move(self):
        object = self.parser.parse("prereport(MakeReservation, [dest_city(paris), dest_city(london)])")
        expected_object = PrereportMove(
            self.ontology_name, "MakeReservation",
            [self.proposition_dest_city_paris, self.proposition_dest_city_london]
        )
        self.assertEquals(expected_object, object)

    def test_create_DoPlanItem(self):
        action = self.parser.parse("buy")
        do_item = self.parser.parse("do(buy)")
        self.assertEquals(action, do_item.getContent())

    def test_create_EmitIcmPlanItem(self):
        icm_move = self.parser.parse("icm:und*pos:USR*goal(perform(buy))")
        item = EmitIcmPlanItem(icm_move)
        self.assertEquals(icm_move, item.getContent())
        self.assertTrue(item.isEmitIcmPlanItem())

    def test_FindoutPlanItem_with_no_params(self):
        expected_item = FindoutPlanItem(self.domain_name, self.dest_city_question)
        item = self.parser.parse("findout(?X.dest_city(X))")
        self.assertEquals(expected_item, item)
        self.assertEquals(self.dest_city_question, item.getContent())

    def test_multiple_parameters(self):
        string = "{graphical_type=list, incremental=True}"
        params = self.parser.parse_parameters(string)
        expected_params = {"graphical_type": "list", "incremental": True}
        self.assertEquals(expected_params, params)

    def test_boolean_parameter_leading_uppercase(self):
        string = "{incremental=True}"
        params = self.parser.parse_parameters(string)
        expected_params = {"incremental": True}
        self.assertEquals(expected_params, params)

    def test_boolean_parameter_lowercase(self):
        string = "{incremental=true}"
        params = self.parser.parse_parameters(string)
        expected_params = {"incremental": True}
        self.assertEquals(expected_params, params)

    def test_verbalize_parameter(self):
        string = "{verbalize=True}"
        params = self.parser.parse_parameters(string)
        expected_params = {"verbalize": True}
        self.assertEquals(expected_params, params)

    def test_parameters_with_empty_alts(self):
        string = "{alts=set([])}"
        params = self.parser.parse_parameters(string)
        expected_params = {"alts": PropositionSet([])}
        self.assertEquals(expected_params, params)

    def test_parameters_with_single_alt(self):
        string = "{alts=set([goal(perform(buy))])}"
        params = self.parser.parse_parameters(string)
        expected_params = {"alts": PropositionSet([self.parser.parse("goal(perform(buy))")])}
        self.assertEquals(expected_params, params)

    def test_parameters_with_multiple_alts(self):
        string = "{alts=set([goal(perform(top)), goal(perform(buy))])}"
        params = self.parser.parse_parameters(string)
        expected_params = {
            "alts": PropositionSet([self.parser.parse("goal(perform(top))"),
                                    self.parser.parse("goal(perform(buy))")])
        }
        self.assertEquals(expected_params, params)

    def test_graphical_type_param_list(self):
        string = "{graphical_type=list}"
        params = self.parser.parse_parameters(string)
        expected_params = {"graphical_type": QuestionRaisingPlanItem.GRAPHICAL_TYPE_LIST}
        self.assertEquals(expected_params, params)

    def test_graphical_type_param_text(self):
        string = "{graphical_type=text}"
        params = self.parser.parse_parameters(string)
        expected_params = {"graphical_type": QuestionRaisingPlanItem.GRAPHICAL_TYPE_TEXT}
        self.assertEquals(expected_params, params)

    def test_source_param_service(self):
        string = "{source=service}"
        params = self.parser.parse_parameters(string)
        expected_params = {"source": QuestionRaisingPlanItem.SOURCE_SERVICE}
        self.assertEquals(expected_params, params)

    def test_source_param_domain(self):
        string = "{source=domain}"
        params = self.parser.parse_parameters(string)
        expected_params = {"source": QuestionRaisingPlanItem.SOURCE_DOMAIN}
        self.assertEquals(expected_params, params)

    def test_service_query_param(self):
        string = "{service_query=?X.available_dest_city(X)}"
        params = self.parser.parse_parameters(string)
        expected_params = {"service_query": self.parser.parse("?X.available_dest_city(X)")}
        self.assertEquals(expected_params, params)

    def test_device_param(self):
        string = "{device=travel_booker}"
        params = self.parser.parse_parameters(string)
        expected_params = {"device": "travel_booker"}
        self.assertEquals(expected_params, params)

    def test_sort_parameter(self):
        string = "{sort_order=alphabetic}"
        params = self.parser.parse_parameters(string)
        expected_params = {"sort_order": QuestionRaisingPlanItem.ALPHABETIC}
        self.assertEquals(expected_params, params)

    def test_background_parameter(self):
        string = "{background=[dest_city, dept_city]}"
        params = self.parser.parse_parameters(string)
        expected_params = {"background": [self.predicate_dest_city, self.predicate_dept_city]}
        self.assertEquals(expected_params, params)

    def test_background_parameter_with_illegal_value(self):
        string = "{background=?X.dest_city(X)}"
        with self.assertRaises(ParseError):
            self.parser.parse_parameters(string)

    def test_allow_goal_accommodation_parameter(self):
        string = "{allow_goal_accommodation=True}"
        params = self.parser.parse_parameters(string)
        expected_params = {"allow_goal_accommodation": True}
        self.assertEquals(expected_params, params)

    def test_max_spoken_alts_parameter(self):
        string = "{max_spoken_alts=2}"
        params = self.parser.parse_parameters(string)
        expected_params = {"max_spoken_alts": 2}
        self.assertEquals(expected_params, params)

    def test_related_information_parameter(self):
        string = "{related_information=[?X.price(X)]}"
        params = self.parser.parse_parameters(string)
        expected_params = {"related_information": [self.price_question]}
        self.assertEquals(expected_params, params)

    def test_ask_features_parameter(self):
        string = "{ask_features=[dest_city, dept_city]}"
        params = self.parser.parse_parameters(string)
        expected_params = {"ask_features": [self.predicate_dest_city, self.predicate_dept_city]}
        self.assertEquals(expected_params, params)

    def test_illegal_parameter_raises_exception(self):
        with self.assertRaises(ParseError):
            self.parser.parse("{sldkfjs=ksjhdf}")

    def test_create_RaisePlanItem(self):
        item = self.parser.parse("raise(?X.dest_city(X))")
        expected_item = RaisePlanItem(self.domain_name, self.dest_city_question)
        self.assertEquals(expected_item, item)

    def test_create_FindoutPlanItem_with_non_question(self):
        with self.assertRaises(ParseError):
            self.parser.parse("findout(X.dest_city(X))")

    def test_create_RaisePlanItem_with_non_question(self):
        with self.assertRaises(ParseError):
            self.parser.parse("raise(X.dest_city(X))")

    def test_create_BindPlanItem(self):
        item = self.parser.parse("bind(?X.dest_city(X))")
        expected_item = BindPlanItem(self.dest_city_question)
        self.assertEquals(expected_item, item)

    def test_create_BindPlanItem_with_non_question(self):
        with self.assertRaises(ParseError):
            self.parser.parse("bind(X.dest_city(X))")

    def test_create_RespondPlanItem(self):
        item = self.parser.parse("respond(?X.dest_city(X))")
        expected_item = RespondPlanItem(self.dest_city_question)
        self.assertEquals(expected_item, item)

    def test_create_ConsultDBPlanItem(self):
        item = self.parser.parse("consultDB(?X.dest_city(X))")
        expected_item = ConsultDBPlanItem(self.dest_city_question)
        self.assertEquals(expected_item, item)

    def test_create_if_then_else(self):
        string = "if dest_city(paris) then jumpto(perform(top)) else jumpto(perform(buy))"
        item = self.parser.parse(string)
        expected_item = IfThenElse(
            self.proposition_dest_city_paris, JumpToPlanItem(PerformGoal(self.top_action)),
            JumpToPlanItem(PerformGoal(self.buy_action))
        )
        self.assertEquals(expected_item, item)

    def test_create_if_then_else_resolvedness_from_string(self):
        string = "if resolved(?X.dest_city(X))" " then jumpto(perform(top))" " else jumpto(perform(buy))"
        item = self.parser.parse(string)

        expected_item = IfThenElse(
            ResolvednessProposition(self.dest_city_question), JumpToPlanItem(PerformGoal(self.top_action)),
            JumpToPlanItem(PerformGoal(self.buy_action))
        )
        self.assertEquals(expected_item, item)

    def test_create_if_then_else_resolvedness_no_else_from_string(self):
        string = "if resolved(?X.dest_city(X))" " then jumpto(perform(top))" " else "
        item = self.parser.parse(string)

        expected_item = IfThenElse(
            ResolvednessProposition(self.dest_city_question), JumpToPlanItem(PerformGoal(self.top_action)), None
        )
        self.assertEquals(expected_item, item)

    def test_create_if_then_else_resolvedness_no_then_from_string(self):
        string = "if resolved(?X.dest_city(X))" " then " " else jumpto(perform(top))"
        item = self.parser.parse(string)

        expected_item = IfThenElse(
            ResolvednessProposition(self.dest_city_question), None, JumpToPlanItem(PerformGoal(self.top_action))
        )
        self.assertEquals(expected_item, item)

    def test_create_forget_all_from_string(self):
        item = self.parser.parse("forget_all")
        self.assertTrue(item.is_forget_all_plan_item())

    def test_create_forget(self):
        fact = self.parser.parse("dest_city(paris)")
        item = self.parser.parse("forget(%s)" % fact)
        self.assertTrue(item.is_forget_plan_item())
        self.assertEqual(fact, item.getContent())

    def test_create_forget_predicate(self):
        predicate = self.parser.parse("dest_city")
        item = self.parser.parse("forget(%s)" % predicate)
        self.assertTrue(item.is_forget_plan_item())
        self.assertEqual(predicate, item.getContent())

    def test_create_forget_issue(self):
        issue = self.parser.parse("?X.dest_city(X)")
        item = self.parser.parse("forget_issue(%s)" % issue)
        self.assertTrue(item.is_forget_issue_plan_item())
        self.assertEqual(issue, item.getContent())

    def test_create_invoke_service_query_plan_item(self):
        issue = self.parser.parse("?X.dest_city(X)")
        item = self.parser.parse("invoke_service_query(%s)" % issue)
        self.assertTrue(item.is_invoke_service_query_plan_item())
        self.assertEqual(issue, item.getContent())
        self.assertEqual(1, item.get_min_results())
        self.assertEqual(1, item.get_max_results())

    def test_create_deprecated_dev_query_plan_item(self):
        issue = self.parser.parse("?X.dest_city(X)")
        item = self.parser.parse("dev_query(%s)" % issue)
        self.assertTrue(item.is_invoke_service_query_plan_item())
        self.assertEqual(issue, item.getContent())
        self.assertEqual(1, item.get_min_results())
        self.assertEqual(1, item.get_max_results())

    def test_invoke_service_action_no_params(self):
        service_action = "MakeReservation"
        item = self.parser.parse("invoke_service_action(%s, {})" % service_action)
        self.assertTrue(item.is_invoke_service_action_plan_item())
        self.assertEqual(service_action, item.get_service_action())

    def test_invoke_service_action_postconfirmed_and_preconfirmed_assertively(self):
        params_string = "{postconfirm=True, preconfirm=assertive}"
        service_action = "MakeReservation"
        item = self.parser.parse("invoke_service_action(%s, %s)" % (service_action, params_string))
        self.assertTrue(item.has_postconfirmation())
        self.assertTrue(item.has_assertive_preconfirmation())

    def test_invoke_service_action_preconfirmed_interrogatively(self):
        params_string = "{preconfirm=interrogative}"
        service_action = "MakeReservation"
        item = self.parser.parse("invoke_service_action(%s, %s)" % (service_action, params_string))
        self.assertTrue(item.has_interrogative_preconfirmation())

    def test_invoke_service_action_downdate_plan_true_by_default(self):
        params_string = "{}"
        service_action = "MakeReservation"
        item = self.parser.parse("invoke_service_action(%s, %s)" % (service_action, params_string))
        self.assertTrue(item.should_downdate_plan())

    def test_invoke_service_action_downdate_plan_overridable(self):
        params_string = "{downdate_plan=False}"
        service_action = "MakeReservation"
        item = self.parser.parse("invoke_service_action(%s, %s)" % (service_action, params_string))
        self.assertFalse(item.should_downdate_plan())

    def test_deprecated_dev_perform_no_params(self):
        service_action = "MakeReservation"
        item = self.parser.parse("dev_perform(%s, {})" % service_action)
        self.assertTrue(item.is_invoke_service_action_plan_item())
        self.assertEqual(service_action, item.get_service_action())

    def test_deprecated_dev_perform_postconfirmed_and_preconfirmed_assertively(self):
        params_string = "{postconfirm=True, preconfirm=assertive}"
        service_action = "MakeReservation"
        item = self.parser.parse("dev_perform(%s, %s)" % (service_action, params_string))
        self.assertTrue(item.has_postconfirmation())
        self.assertTrue(item.has_assertive_preconfirmation())

    def test_deprecated_dev_perform_preconfirmed_interrogatively(self):
        params_string = "{preconfirm=interrogative}"
        service_action = "MakeReservation"
        item = self.parser.parse("dev_perform(%s, %s)" % (service_action, params_string))
        self.assertTrue(item.has_interrogative_preconfirmation())

    def test_create_yesno_question(self):
        question = self.parser.parse("?dest_city(paris)")
        self.assertTrue(question.is_yes_no_question())

    def test_create_single_alt_question(self):
        alt_question_string = "?set([goal(perform(buy))])"
        question = self.parser.parse(alt_question_string)
        self.assertEquals(alt_question_string, unicode(question))

    def test_create_alt_question_with_predicate_propositions(self):
        alt_question_string = "?set([goal(perform(top)), goal(perform(buy))])"
        question = self.parser.parse(alt_question_string)
        self.assertEquals(alt_question_string, unicode(question))

    def test_create_alt_question_with_goal_propositions(self):
        alt_question_string = "?set([goal(perform(buy)), goal(perform(top))])"
        question = self.parser.parse(alt_question_string)
        self.assertEquals(alt_question_string, unicode(question))

    def test_create_single_alt_question_with_issue_proposition(self):
        alt_question_string = "?set([goal(resolve(?X.goal(X)))])"
        question = self.parser.parse(alt_question_string)
        self.assertEquals(alt_question_string, unicode(question))

    def test_create_single_neg_alt_question(self):
        alt_question_string = "?set([~goal(perform(buy))])"
        question = self.parser.parse(alt_question_string)
        self.assertEquals(alt_question_string, unicode(question))

    def test_is_alt_question_when_created(self):
        question_as_string = "?set([goal(perform(top)), goal(perform(buy))])"
        question = self.parser.parse(question_as_string)
        self.assertTrue(question.is_alt_question())

    def test_proposition_set_with_action_and_issue_propositions(self):
        string = "set([goal(perform(buy)), goal(resolve(?X.price(X)))])"
        object = self.parser.parse(string)
        expected_object = PropositionSet([
            GoalProposition(PerformGoal(self.buy_action)),
            GoalProposition(ResolveGoal(self.price_question, Speaker.SYS))
        ])
        self.assertEquals(expected_object, object)

    def test_negative_proposition_set_with_action_and_issue_propositions(self):
        string = "~set([goal(perform(buy)), goal(resolve(?X.price(X)))])"
        object = self.parser.parse(string)
        expected_object = PropositionSet([
            GoalProposition(PerformGoal(self.buy_action)),
            GoalProposition(ResolveGoal(self.price_question, Speaker.SYS))
        ],
                                         polarity=Polarity.NEG)
        self.assertEquals(expected_object, object)

    def test_greet_move(self):
        move = self.parser.parse("greet")
        self.assertEquals(GreetMove(), move)

    def test_user_greet_move_with_speaker_and_score(self):
        move = self.parser.parse("Move(greet, ddd_name='mockup_ddd', speaker=USR, understanding_confidence=0.47)")
        self.assertEquals(Speaker.USR, move.get_speaker())
        self.assertEquals(0.47, move.understanding_confidence)

    def test_mute_move(self):
        move = self.parser.parse("mute")
        self.assertEquals(MuteMove(), move)

    def test_sys_mute_move_with_score(self):
        move = self.parser.parse("Move(mute, speaker=SYS, understanding_confidence=1.0)")
        self.assertEquals(Speaker.SYS, move.get_speaker())
        self.assertEquals(1.0, move.understanding_confidence)

    def test_move_without_score(self):
        move = self.parser.parse("Move(greet, speaker=SYS)")
        self.assertEquals(Speaker.SYS, move.get_speaker())

    def test_unmute_move(self):
        move = self.parser.parse("unmute")
        self.assertEquals(Move.UNMUTE, move.get_type())

    def test_quit_move(self):
        move = self.parser.parse("quit")
        self.assertEquals(Move.QUIT, move.get_type())

    def test_contentful_reraise(self):
        move = self.parser.parse("icm:reraise:top")
        self.assertEquals(ICMMove.RERAISE, move.get_type())
        self.assertEquals(self.top_action, move.get_content())

    def test_contentless_reraise(self):
        move = self.parser.parse("icm:reraise")
        self.assertEquals(ICMMove.RERAISE, move.get_type())

    def test_reraise_with_speaker_and_score(self):
        speaker = Speaker.SYS
        score = 1.0
        move = self.parser.parse("ICMMove(icm:reraise:top, speaker=%s, understanding_confidence=%s)" % (speaker, score))
        expected_move = ICMMoveWithSemanticContent(
            ICMMove.RERAISE, self.top_action, understanding_confidence=score, speaker=speaker
        )
        self.assertEquals(expected_move, move)

    def test_resume(self):
        move = self.parser.parse("icm:resume:perform(buy)")
        self.assertEquals(ICMMove.RESUME, move.get_type())
        self.assertEquals(PerformGoal(self.buy_action), move.get_content())

    def test_contentful_accommodate(self):
        move = self.parser.parse("icm:accommodate:top")
        self.assertEquals(ICMMove.ACCOMMODATE, move.get_type())
        self.assertEquals(self.top_action, move.get_content())

    def test_contentless_accommodate(self):
        move = self.parser.parse("icm:accommodate")
        self.assertEquals(ICMMove.ACCOMMODATE, move.get_type())

    def test_loadplan(self):
        move = self.parser.parse("icm:loadplan")
        self.assertTrue(move.get_type() == ICMMove.LOADPLAN)

    def test_und_neg_without_content(self):
        icm = self.parser.parse("icm:und*neg")
        self.assertEquals(ICMMove.UND, icm.get_type())
        self.assertEquals(ICMMove.NEG, icm.get_polarity())
        self.assertEquals(None, icm.get_content())

    def test_sem_neg(self):
        icm = self.parser.parse("icm:sem*neg")
        self.assertEquals(ICMMove.SEM, icm.get_type())
        self.assertEquals(ICMMove.NEG, icm.get_polarity())
        self.assertEquals(None, icm.get_content())

    def test_contentful_per_pos(self):
        icm = self.parser.parse('icm:per*pos:"a string"')
        self.assertEquals(ICMMove.PER, icm.get_type())
        self.assertEquals(ICMMove.POS, icm.get_polarity())
        expected_content = 'a string'
        self.assertEquals(expected_content, icm.get_content())

    def test_contentful_per_pos_from_stringfyed_move(self):
        move = ICMMoveWithStringContent(ICMMove.PER, content="a string", polarity=ICMMove.POS)
        icm = self.parser.parse(unicode(move))
        self.assertEquals(ICMMove.PER, icm.get_type())
        self.assertEquals(ICMMove.POS, icm.get_polarity())
        expected_content = "a string"
        self.assertEquals(expected_content, icm.get_content())

    def test_und_int_usr(self):
        icm = self.parser.parse("icm:und*int:USR*dest_city(paris)")
        self.assertEquals(ICMMove.UND, icm.get_type())
        self.assertEquals(ICMMove.INT, icm.get_polarity())
        self.assertEquals("dest_city(paris)", unicode(icm.get_content()))

    def test_und_int_model(self):
        icm = self.parser.parse("icm:und*int:MODEL*dest_city(paris)")
        self.assertEquals(ICMMove.UND, icm.get_type())
        self.assertEquals(ICMMove.INT, icm.get_polarity())
        self.assertEquals(Speaker.MODEL, icm.get_content_speaker())
        self.assertEquals("dest_city(paris)", unicode(icm.get_content()))

    def test_und_int_w_speaker_and_score(self):
        speaker = Speaker.SYS
        score = 1.0
        icm = self.parser.parse(
            "ICMMove(icm:und*int:USR*dest_city(paris), speaker=%s, understanding_confidence=%s)" % (speaker, score)
        )
        content = self.parser.parse("dest_city(paris)")
        expected_move = ICMMoveWithSemanticContent(
            ICMMove.UND,
            content,
            understanding_confidence=score,
            speaker=speaker,
            content_speaker=Speaker.USR,
            polarity=ICMMove.INT
        )
        self.assertEquals(expected_move, icm)

    def test_acc_pos_w_speaker_and_no_score(self):
        speaker = Speaker.SYS
        icm = self.parser.parse("ICMMove(icm:acc*pos, speaker=%s)" % speaker)
        expected_move = ICMMove(ICMMove.ACC, understanding_confidence=1.0, speaker=speaker, polarity=ICMMove.POS)
        self.assertEquals(expected_move, icm)

    def test_und_int_icm_fails_if_faulty_speaker(self):
        with self.assertRaises(Exception):
            self.parser.parse("icm:und*int:usr*dest_city(paris)")

    def test_und_pos_icm_for_issue_proposition(self):
        icm = self.parser.parse("icm:und*pos:USR*goal(resolve(?X.dest_city(X)))")
        self.assertEquals(ICMMove.UND, icm.get_type())
        self.assertEquals(ICMMove.POS, icm.get_polarity())
        self.assertEquals("goal(resolve(?X.dest_city(X)))", unicode(icm.get_content()))

    def test_und_int_icm_without_speaker(self):
        score = 1.0
        speaker = Speaker.SYS
        icm = self.parser.parse(
            "ICMMove(icm:und*int:dest_city(paris), speaker=%s, understanding_confidence=%s)" % (speaker, score)
        )
        content = self.parser.parse("dest_city(paris)")
        expected_move = ICMMoveWithSemanticContent(
            ICMMove.UND,
            content,
            understanding_confidence=score,
            speaker=speaker,
            content_speaker=None,
            polarity=ICMMove.INT
        )
        self.assertEquals(expected_move, icm)

    def test_und_pos_icm_for_predicate_proposition(self):
        icm = self.parser.parse("icm:und*pos:USR*dest_city(paris)")
        self.assertEquals(ICMMove.UND, icm.get_type())
        self.assertEquals(ICMMove.POS, icm.get_polarity())
        self.assertEquals("dest_city(paris)", unicode(icm.get_content()))

    def test_per_neg_icm(self):
        icm = self.parser.parse("icm:per*neg")
        self.assertEquals(ICMMove.PER, icm.get_type())
        self.assertEquals(ICMMove.NEG, icm.get_polarity())

    def test_per_neg_icm_w_speaker_and_score(self):
        speaker = Speaker.SYS
        score = 1.0
        icm = self.parser.parse("ICMMove(icm:per*neg, speaker=%s, understanding_confidence=%s)" % (speaker, score))
        expected_move = ICMMove(ICMMove.PER, understanding_confidence=score, speaker=speaker, polarity=ICMMove.NEG)
        self.assertEquals(expected_move, icm)

    def test_acc_pos_icm_no_content(self):
        icm = self.parser.parse("icm:acc*pos")
        self.assertEquals(ICMMove.ACC, icm.get_type())
        self.assertEquals(ICMMove.POS, icm.get_polarity())

    def test_acc_neg_icm_for_issue(self):
        icm = self.parser.parse("icm:acc*neg:issue")
        self.assertEquals(ICMMove.ACC, icm.get_type())
        self.assertEquals(ICMMove.NEG, icm.get_polarity())
        self.assertEquals(str(icm), "ICMMove(icm:acc*neg:issue)")

    def test_unsupported_icm_content_unparseable(self):
        with self.assertRaises(ParseError):
            self.parser.parse("icm:acc*neg:unsupported")

    def test_acc_neg_issue_w_speaker_and_score(self):
        speaker = Speaker.SYS
        score = 1.0
        icm = self.parser.parse(
            "ICMMove(icm:acc*neg:issue, speaker=%s, understanding_confidence=%s)" % (speaker, score)
        )
        expected_move = IssueICMMove(ICMMove.ACC, understanding_confidence=score, speaker=speaker, polarity=ICMMove.NEG)
        self.assertEquals(expected_move, icm)

    def test_undecorated_icm_move_with_ICMMove_classname(self):
        string = 'ICMMove(icm:per*pos:"this is not understood")'
        actual_move = self.parser.parse(string)
        expected_move = ICMMoveWithStringContent(ICMMove.PER, "this is not understood", polarity=ICMMove.POS)
        self.assertEquals(expected_move, actual_move)

    def test_jumpto(self):
        item = self.parser.parse("jumpto(perform(buy))")
        expected_item = JumpToPlanItem(PerformGoal(self.buy_action))
        self.assertEquals(expected_item, item)

    def test_assume(self):
        item = self.parser.parse("assume(dest_city(paris))")
        expected_item = AssumePlanItem(self.proposition_dest_city_paris)
        self.assertEquals(expected_item, item)

    def test_assume_shared(self):
        item = self.parser.parse("assume_shared(dest_city(paris))")
        expected_item = AssumeSharedPlanItem(self.proposition_dest_city_paris)
        self.assertEquals(expected_item, item)

    def test_assume_issue(self):
        item = self.parser.parse("assume_issue(?X.dest_city(X))")
        expected_item = AssumeIssuePlanItem(self.dest_city_question)
        self.assertEquals(expected_item, item)

    def test_service_action_terminated_proposition(self):
        object = self.parser.parse("service_action_terminated(SomeServiceAction)")
        expected_object = ServiceActionTerminatedProposition(self.ontology_name, "SomeServiceAction")
        self.assertEquals(expected_object, object)

    def test_service_action_started_proposition(self):
        object = self.parser.parse("service_action_started(SomeServiceAction)")
        expected_object = ServiceActionStartedProposition(self.ontology_name, "SomeServiceAction")
        self.assertEquals(expected_object, object)

    def test_string_individual(self):
        object = self.parser.parse('"a string"')
        expected_object = Individual(self.ontology_name, 'a string', StringSort())
        self.assertEquals(expected_object, object)

    def test_string_individual_digits_only(self):
        object = self.parser.parse('"123"')
        expected_object = Individual(self.ontology_name, '123', StringSort())
        self.assertEquals(expected_object, object)

    def test_string_answer(self):
        move = self.parser.parse('answer("a string")')
        individual = Individual(self.ontology_name, 'a string', StringSort())
        expected_move = self._move_factory.createAnswerMove(individual)
        self.assertEquals(expected_move, move)

    def test_predicate_proposition_with_string_individual(self):
        object = self.parser.parse('number_to_call("070123456")')
        expected_object = PredicateProposition(
            self.predicate_number_to_call, self.ontology.create_individual('"070123456"')
        )
        self.assertEquals(expected_object, object)

    def test_unknown_term_unparseable(self):
        with self.assertRaises(ParseError):
            self.parser.parse("unknown_term")

    def test_parse_creates_new_instance(self):
        container_string = "{}"
        container = self.parser.parse(container_string)
        container.add("dummy_item")
        new_container = self.parser.parse(container_string)
        expected_new_container = Set()
        self.assertEquals(expected_new_container, new_container)

    def test_perform_goal(self):
        string = "perform(top)"
        expected_object = PerformGoal(self.top_action)
        self.assertEquals(expected_object, self.parser.parse(string))

    def test_resolve_goal(self):
        string = "resolve(?X.price(X))"
        expected_object = ResolveGoal(self.price_question, Speaker.SYS)
        self.assertEquals(expected_object, self.parser.parse(string))

    def test_resolve_user_goal(self):
        object = self.parser.parse("resolve_user(?X.dest_city(X))")
        expected_object = ResolveGoal(self.dest_city_question, Speaker.USR)
        self.assertEquals(expected_object, object)

    def test_handle_goal(self):
        string = "handle(ReservationConfirmed)"
        expected_object = HandleGoal(self.ontology_name, "ReservationConfirmed")
        self.assertEquals(expected_object, self.parser.parse(string))

    def test_deprecated_action_question_gives_helpful_error(self):
        string = "?X.action(X)"
        with self.assertRaises(ParseError) as cm:
            self.parser.parse(string)
        self.assertEquals(
            "'?X.action(X)' is not a valid question. Perhaps you mean '?X.goal(X)'.", unicode(cm.exception)
        )

    def test_deprecated_action_proposition_gives_helpful_error(self):
        string = "set([action(make_domestic_reservation), action(make_international_reservation)])"
        with self.assertRaises(ParseError) as cm:
            self.parser.parse(string)
        self.assertEquals(
            "'action(make_domestic_reservation)' is not a valid proposition. Perhaps you mean 'goal(perform(make_domestic_reservation))'.",
            unicode(cm.exception)
        )

    def test_deprecated_issue_proposition_gives_helpful_error(self):
        string = "set([issue(?X.price(X))])"
        with self.assertRaises(ParseError) as cm:
            self.parser.parse(string)
        self.assertEquals(
            "'issue(?X.price(X))' is not a valid proposition. Perhaps you mean 'goal(resolve(?X.price(X)))'.",
            unicode(cm.exception)
        )

    def test_predicate(self):
        string = "dest_city"
        expected_object = self.predicate_dest_city
        self.assertEquals(expected_object, self.parser.parse(string))

    def test_successful_perform_result(self):
        string = "SuccessfulServiceAction()"
        expected_object = SuccessfulServiceAction()
        self.assertEquals(expected_object, self.parser.parse(string))

    def test_failed_perform_result(self):
        string = "FailedServiceAction(mock_failure_reason)"
        expected_object = FailedServiceAction("mock_failure_reason")
        self.assertEquals(expected_object, self.parser.parse(string))



from tala.model.move import ICMMove
from tala.model.proposition import Proposition
from tala.model.semantic_object import SemanticObject, OntologySpecificSemanticObject, SemanticObjectWithContent
from tala.utils.as_json import convert_to_json


class PlanItem(SemanticObject):
    TYPE_RESPOND = "respond"
    TYPE_GREET = "greet"
    TYPE_QUIT = "quit"
    TYPE_MUTE = "mute"
    TYPE_UNMUTE = "unmute"
    TYPE_FINDOUT = "findout"
    TYPE_CONSULTDB = "consultDB"
    TYPE_RAISE = "raise"
    TYPE_BIND = "bind"
    TYPE_DO = "do"
    TYPE_EMIT_ICM = "emit_icm"
    TYPE_JUMPTO = "jumpto"
    TYPE_IF_THEN_ELSE = "if_then_else"
    TYPE_FORGET_ALL = "forget_all"
    TYPE_FORGET = "forget"
    TYPE_FORGET_ISSUE = "forget_issue"
    TYPE_INVOKE_SERVICE_QUERY = "invoke_service_query"
    TYPE_INVOKE_SERVICE_ACTION = "invoke_service_action"
    TYPE_SERVICE_REPORT = "service_report"
    TYPE_ASSUME = "assume"
    TYPE_ASSUME_SHARED = "assume_shared"
    TYPE_ASSUME_ISSUE = "assume_issue"
    TYPE_EMIT_MOVE = "emit_move"
    TYPE_HANDLE = "handle"

    def __init__(self, type):
        SemanticObject.__init__(self)
        self._type = type

    def __repr__(self):
        return "%s%s" % (PlanItem.__class__.__name__, (self._type, ))

    def __str__(self):
        return unicode(self._type)

    def __eq__(self, other):
        return other.is_plan_item() and self.get_type() == other.get_type()

    def __ne__(self, other):
        return not (self == other)

    def __hash__(self):
        return hash((self.__class__.__name__, self._type))

    def get_type(self):
        return self._type

    def is_plan_item(self):
        return True

    def isRespondPlanItem(self):
        return self._type == self.TYPE_RESPOND

    def isQuitPlanItem(self):
        return self._type == self.TYPE_QUIT

    def is_mute_plan_item(self):
        return self._type == self.TYPE_MUTE

    def is_unmute_plan_item(self):
        return self._type == self.TYPE_UNMUTE

    def isGreetPlanItem(self):
        return self._type == self.TYPE_GREET

    def isFindoutPlanItem(self):
        return self._type == self.TYPE_FINDOUT

    def isConsultDBPlanItem(self):
        return self._type == self.TYPE_CONSULTDB

    def isRaisePlanItem(self):
        return self._type == self.TYPE_RAISE

    def isBindPlanItem(self):
        return self._type == self.TYPE_BIND

    def isDoPlanItem(self):
        return self._type == self.TYPE_DO

    def isEmitIcmPlanItem(self):
        return self._type == self.TYPE_EMIT_ICM

    def is_question_plan_item(self):
        return self._type in [self.TYPE_FINDOUT, self.TYPE_RAISE, self.TYPE_BIND]

    def is_forget_all_plan_item(self):
        return self._type == self.TYPE_FORGET_ALL

    def is_forget_plan_item(self):
        return self._type == self.TYPE_FORGET

    def is_forget_issue_plan_item(self):
        return self._type == self.TYPE_FORGET_ISSUE

    def is_invoke_service_query_plan_item(self):
        return self._type == self.TYPE_INVOKE_SERVICE_QUERY

    def is_invoke_service_action_plan_item(self):
        return self._type == self.TYPE_INVOKE_SERVICE_ACTION

    def is_service_report_plan_item(self):
        return self._type == self.TYPE_SERVICE_REPORT

    def is_jumpto_plan_item(self):
        return self._type == self.TYPE_JUMPTO

    def is_assume_plan_item(self):
        return self._type == self.TYPE_ASSUME

    def is_assume_shared_plan_item(self):
        return self._type == self.TYPE_ASSUME_SHARED

    def is_assume_issue_plan_item(self):
        return self._type == self.TYPE_ASSUME_ISSUE

    def is_emit_move_plan_item(self):
        return self._type == self.TYPE_EMIT_MOVE

    def is_handle_plan_item(self):
        return self._type == self.TYPE_HANDLE

    def getType(self):
        return self._type

    def is_question_raising_item(self):
        return False

    def is_turn_yielding(self):
        return False

    def as_json(self):
        return {
            self.get_type(): None,
        }


class PlanItemWithSemanticContent(PlanItem, SemanticObjectWithContent):
    def __init__(self, type, content):
        PlanItem.__init__(self, type)
        SemanticObjectWithContent.__init__(self, content)
        self._content = content

    def __repr__(self):
        return "%s%s" % (PlanItemWithSemanticContent.__name__, (self._type, self._content))

    def __str__(self):
        return "%s(%s)" % (unicode(self._type), unicode(self._content))

    def __eq__(self, other):
        return other is not None and other.is_plan_item() and other.has_semantic_content() and self.get_type(
        ) == other.get_type() and self.get_content() == other.get_content()

    def __ne__(self, other):
        return not (self == other)

    def __hash__(self):
        return hash((self.__class__.__name__, self._type, self._content))

    def get_type(self):
        return self._type

    def get_content(self):
        return self._content

    def getContent(self):
        return self.get_content()

    def as_json(self):
        return {self.get_type(): convert_to_json(self._content)}


class AssumePlanItem(PlanItemWithSemanticContent):
    def __init__(self, content):
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_ASSUME, content=content)


class AssumeSharedPlanItem(PlanItemWithSemanticContent):
    def __init__(self, content):
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_ASSUME_SHARED, content=content)


class AssumeIssuePlanItem(PlanItemWithSemanticContent):
    def __init__(self, content):
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_ASSUME_ISSUE, content=content)


class RespondPlanItem(PlanItemWithSemanticContent):
    def __init__(self, content):
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_RESPOND, content=content)

    def is_turn_yielding(self):
        return True


class DoPlanItem(PlanItemWithSemanticContent):
    def __init__(self, action):
        assert action.is_action()
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_DO, content=action)


class QuitPlanItem(PlanItem):
    def __init__(self):
        PlanItem.__init__(self, PlanItem.TYPE_QUIT)


class MutePlanItem(PlanItem):
    def __init__(self):
        PlanItem.__init__(self, PlanItem.TYPE_MUTE)


class UnmutePlanItem(PlanItem):
    def __init__(self):
        PlanItem.__init__(self, PlanItem.TYPE_UNMUTE)


class GreetPlanItem(PlanItem):
    def __init__(self):
        PlanItem.__init__(self, PlanItem.TYPE_GREET)


class EmitIcmPlanItem(PlanItemWithSemanticContent):
    def __init__(self, icm_move):
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_EMIT_ICM, content=icm_move)

    def is_question_raising_item(self):
        icm = self.getContent()
        return (
            icm.get_type() == ICMMove.UND
            and not (icm.get_polarity() == ICMMove.POS and not icm.get_content().is_positive())
        )

    def is_turn_yielding(self):
        icm = self.getContent()
        return (icm.get_type() == ICMMove.ACC and icm.get_polarity() == ICMMove.NEG)


class BindPlanItem(PlanItemWithSemanticContent):
    def __init__(self, content):
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_BIND, content)

    def get_question(self):
        return self.getContent()


class ConsultDBPlanItem(PlanItemWithSemanticContent):
    def __init__(self, content):
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_CONSULTDB, content=content)


class JumpToPlanItem(PlanItemWithSemanticContent):
    def __init__(self, content):
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_JUMPTO, content=content)


class IfThenElse(PlanItem):
    def __init__(self, condition, consequent, alternative):
        self.condition = condition
        self.consequent = consequent
        self.alternative = alternative
        assert consequent is not None or alternative is not None, "One of consequent (%s) and alternative (%s) must not be None" % (
            consequent, alternative
        )
        if consequent is not None and alternative is not None:
            assert consequent.ontology_name == alternative.ontology_name, "Expected identical ontologies in both consequent (%s) and alternative (%s) but got %r and %r" % (
                consequent, alternative, consequent.ontology_name, alternative.ontology_name
            )
        PlanItem.__init__(self, PlanItem.TYPE_IF_THEN_ELSE)

    def get_condition(self):
        return self.condition

    def get_consequent(self):
        return self.consequent

    def get_alternative(self):
        return self.alternative

    def remove_consequent(self):
        self.consequent = None

    def remove_alternative(self):
        self.alternative = None

    def __str__(self):
        return "if_then_else(%s, %s, %s)" % (self.condition, self.consequent, self.alternative)

    def as_json(self):
        return {
            self.get_type(): {
                "condition": convert_to_json(self.condition),
                "consequent": convert_to_json(self.consequent),
                "alternative": convert_to_json(self.alternative),
            }
        }


class ForgetAllPlanItem(PlanItem):
    def __init__(self):
        PlanItem.__init__(self, PlanItem.TYPE_FORGET_ALL)


class ForgetPlanItem(PlanItemWithSemanticContent):
    def __init__(self, predicate_or_proposition):
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_FORGET, predicate_or_proposition)


class ForgetIssuePlanItem(PlanItemWithSemanticContent):
    def __init__(self, issue):
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_FORGET_ISSUE, issue)


class MinResultsNotSupportedException(Exception):
    pass


class MaxResultsNotSupportedException(Exception):
    pass


class InvokeServiceQueryPlanItem(PlanItemWithSemanticContent):
    def __init__(self, issue, min_results=None, max_results=None):
        min_results = min_results or 0
        if min_results < 0:
            raise MinResultsNotSupportedException("Expected 'min_results' to be 0 or above but got %r." % min_results)
        if max_results is not None and max_results < 1:
            raise MaxResultsNotSupportedException(
                "Expected 'max_results' to be None or above 0 but got %r." % max_results
            )
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_INVOKE_SERVICE_QUERY, issue)
        self._min_results = min_results
        self._max_results = max_results

    def get_min_results(self):
        return self._min_results

    def get_max_results(self):
        return self._max_results

    def __str__(self):
        return "invoke_service_query(%s, min_results=%s, max_results=%s)" % (
            self._content, self._min_results, self._max_results
        )

    def __repr__(self):
        return "%s(%r, min_results=%r, max_results=%r)" % (
            self.__class__.__name__, self._content, self._min_results, self._max_results
        )

    def __eq__(self, other):
        return super(PlanItemWithSemanticContent, self).__eq__(other) and other.get_min_results(
        ) == self.get_min_results() and other.get_max_results() == self.get_max_results()

    def as_json(self):
        return {
            self.get_type(): {
                "issue": convert_to_json(self._content),
                "min_results": self._min_results,
                "max_results": self._max_results,
            }
        }


class InvokeServiceActionPlanItem(PlanItem, OntologySpecificSemanticObject):
    INTERROGATIVE = "INTERROGATIVE"
    ASSERTIVE = "ASSERTIVE"

    def __init__(self, ontology_name, service_action, preconfirm=None, postconfirm=False, downdate_plan=True):
        self.service_action = service_action
        self.preconfirm = preconfirm
        self.postconfirm = postconfirm
        self._downdate_plan = downdate_plan
        PlanItem.__init__(self, PlanItem.TYPE_INVOKE_SERVICE_ACTION)
        OntologySpecificSemanticObject.__init__(self, ontology_name)

    def get_service_action(self):
        return self.service_action

    def has_interrogative_preconfirmation(self):
        return self.preconfirm == self.INTERROGATIVE

    def has_assertive_preconfirmation(self):
        return self.preconfirm == self.ASSERTIVE

    def has_postconfirmation(self):
        return self.postconfirm

    def should_downdate_plan(self):
        return self._downdate_plan

    def __eq__(self, other):
        return super(InvokeServiceActionPlanItem, self).__eq__(other) and other.get_service_action(
        ) == self.get_service_action() and other.has_interrogative_preconfirmation(
        ) == self.has_interrogative_preconfirmation() and other.has_assertive_preconfirmation(
        ) == self.has_assertive_preconfirmation() and other.has_postconfirmation() == self.has_postconfirmation(
        ) and other.should_downdate_plan() == self.should_downdate_plan()

    def __str__(self):
        return "invoke_service_action(%s, {preconfirm=%s, postconfirm=%s, downdate_plan=%s})" % (
            self.service_action, self.preconfirm, self.postconfirm, self._downdate_plan
        )

    def __repr__(self):
        return "%s(%r, %r, preconfirm=%r, postconfirm=%r, downdate_plan=%r)" % (
            self.__class__.__name__, self.ontology_name, self.service_action, self.preconfirm, self.postconfirm,
            self._downdate_plan
        )

    def as_json(self):
        return {
            self.get_type(): {
                "service_action": convert_to_json(self.service_action),
                "ontology": convert_to_json(self.ontology_name),
                "preconfirm": convert_to_json(self.preconfirm),
                "postconfirm": convert_to_json(self.postconfirm),
                "downdate_plan": convert_to_json(self._downdate_plan),
            }
        }


class ServiceReportPlanItem(PlanItemWithSemanticContent):
    def __init__(self, result_proposition):
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_SERVICE_REPORT, result_proposition)

    def is_turn_yielding(self):
        return self._content.get_type() == Proposition.SERVICE_RESULT


class EmitMovePlanItem(PlanItemWithSemanticContent):
    def __init__(self, move):
        PlanItemWithSemanticContent.__init__(self, PlanItem.TYPE_EMIT_MOVE, move)


class HandlePlanItem(PlanItem, OntologySpecificSemanticObject):
    def __init__(self, ontology_name, service_action):
        PlanItem.__init__(self, PlanItem.TYPE_HANDLE)
        OntologySpecificSemanticObject.__init__(self, ontology_name)
        self._service_action = service_action

    @property
    def service_action(self):
        return self._service_action

    def as_json(self):
        return {
            self.get_type(): {
                "service_action": convert_to_json(self._service_action),
                "ontology": convert_to_json(self.ontology_name),
            }
        }



from copy import copy
import os
import shutil
import sys
import tempfile
import unittest

from tala.config import BackendConfig, DddConfig


class DddMockingTestCase(unittest.TestCase):
    def setUp(self):
        self._working_dir = os.getcwd()
        temp_dir = tempfile.mkdtemp(prefix="DddMockingTestCase")
        os.chdir(temp_dir)
        self._temp_dir = os.getcwd()
        self._exception_occurred = False
        self._mock_ddd_config = DddConfig.default_config()
        self._mock_backend_config = BackendConfig.default_config()

    def tearDown(self):
        os.chdir(self._working_dir)
        if self._exception_occurred:
            if "--keep-temp-dir" in sys.argv:
                print "NOTE: Temp dir %s kept for debugging. It needs to be deleted manually." % self._temp_dir
            else:
                self._delete_temp_dir()
                print "NOTE: Temp dir deleted. To keep the temp dir for debugging, " "run again with --keep-temp-dir"
        else:
            self._delete_temp_dir()

    def _delete_temp_dir(self):
        shutil.rmtree(self._temp_dir)

    def _given_ontology_py_file(self, path):
        content = """
from tala.model.ontology import DddOntology
class MockupOntology(DddOntology):
  sorts = {}
  predicates = {}
  individuals = {}
  actions = set([])
"""
        self.create_mockup_file(path, content)

    def _given_domain_py_file(self, path):
        content = """
from tala.model.domain import DddDomain
class MockupDomain(DddDomain):
  plans = []
"""
        self.create_mockup_file(path, content)

    def create_mockup_domain_file(self, path="mockup_app/domain.xml"):
        content = """
<domain name="MockupDomain"/>
"""
        self.create_mockup_file(path, content)

    def _given_domain_xml_file(self, path, domain_name="MockupDomain"):
        content = '<domain name="%s"/>' % domain_name
        self.create_mockup_file(path, content)

    def _given_service_interface_xml_file(self, path):
        content = '<service_interface/>'
        self.create_mockup_file(path, content)

    def _given_ontology_xml_file(self, path):
        content = '<ontology name="MockupOntology"/>'
        self.create_mockup_file(path, content)

    def _given_device_py_file(self, *args, **kwargs):
        self.create_mockup_device_file(*args, **kwargs)

    def create_mockup_device_file(self, path="mockup_app/device.py", content=None):
        if content is None:
            content = """
from tala.model.device import DddDevice
class MockupDevice(DddDevice):
  pass
"""
        self.create_mockup_file(path, content)

    def _given_mocked_ddd_config(
        self, use_rgl=False, use_third_party_parser=False, device_module="device.py", enable_rasa_nlu=False
    ):
        default_config = DddConfig.default_config()
        overrides = {
            "use_rgl": use_rgl,
            "use_third_party_parser": use_third_party_parser,
            "device_module": device_module,
            "enable_rasa_nlu": enable_rasa_nlu,
        }
        config = self._override_default_config(default_config, overrides)
        self._mock_ddd_config = config

    def _given_mocked_backend_config(self, ddd, language_codes=["eng"]):
        default_config = BackendConfig.default_config()
        overrides = {
            "ddds": [ddd],
            "active_ddd": ddd,
            "supported_languages": language_codes,
        }
        config = self._override_default_config(default_config, overrides)
        self._mock_backend_config = config

    def _override_default_config(self, default_config, overrides):
        config = copy(default_config)
        config.update(overrides)
        return config

    def create_mockup_file(self, relative_path, content):
        container_directory = os.path.dirname(relative_path)
        self._ensure_dir_exists(container_directory)
        with open(relative_path, "w") as f:
            f.write(content)

    def _ensure_dir_exists(self, path):
        if not os.path.exists(path):
            os.makedirs(path)

    def _given_third_party_parser_file(self, path):
        content = """
from tala.model.third_party_parser import ThirdPartyParser
class MockThirdPartyParser(ThirdPartyParser):
    name = "MockThirdPartyParser"
    def parse(self, string):
        return "parsed"
"""
        self.create_mockup_file(path, content)



# -*- coding: utf-8 -*-

from contextlib import contextmanager
import os
import warnings

from mock import Mock, patch
import pytest

from tala.model.ddd import DDD
from tala.model.grammar import grammar
from tala.model.grammar.grammar import Grammar, GrammarForRGL, UnexpectedAnswerFormatException
from tala.ddd.grammar.parser import GrammarParser
from tala.model.grammar.intent import Question, Request, Answer
from tala.model.grammar.required_entity import RequiredPropositionalEntity, RequiredSortalEntity
from tala.model.ontology import Ontology
from tala.model.sort import Sort


class TestGrammar(object):
    def setup_method(self):
        self._mocked_ddd = self._create_mocked_ddd()
        self._grammar = None
        self._grammar_path = None
        self._result = None

    def _create_mocked_ddd(self):
        ontology = Mock(spec=Ontology)
        ontology.get_individuals_of_sort.return_value = []
        ddd = Mock(spec=DDD)
        ddd.ontology = ontology
        return ddd

    @pytest.mark.parametrize(
        "GrammarClass,grammar", [(GrammarForRGL, "tala/model/grammar/test/grammar_for_rgl_example.xml"),
                                 (Grammar, "tala/model/grammar/test/grammar_example.xml")]
    )
    def test_requests_of_action(self, GrammarClass, grammar):
        self.given_ddd_name("rasa_test")
        self.given_individuals_in_ontology(
            sort="contact",
            individuals=[
                "contact_john",
                "contact_john_chi",
                "contact_lisa",
                "contact_mary",
                "contact_andy",
                "contact_andy_chi",
            ]
        )
        self.given_grammar_file(grammar)
        self.given_grammar_from_class(GrammarClass)
        self.when_fetching_requests_of_action("call")
        self.then_result_is([
            Request("call", ["make a call"], []),
            Request("call", ["call ", ""], [
                RequiredSortalEntity("contact"),
            ]),
            Request("call", ["make a call to ", ""], [
                RequiredPropositionalEntity("selected_contact_to_call"),
            ]),
        ])

    def given_ddd_name(self, name):
        self._mocked_ddd.name = name

    def given_individuals_in_ontology(self, sort, individuals):
        mocked_sort = Mock(spec=Sort)
        mocked_sort.get_name.return_value = sort
        self._mocked_ddd.ontology.individual_sort.return_value = mocked_sort
        self._mocked_ddd.ontology.get_individuals_of_sort.return_value = individuals

    def given_grammar_file(self, path):
        absolute_path = os.path.abspath(path)
        self._grammar_path = absolute_path

    def given_grammar_from_class(self, GrammarClass):
        with open(self._grammar_path, 'r') as grammar_file:
            grammar_string = grammar_file.read()
        grammar_root = GrammarParser.parse(grammar_string)
        self._grammar = GrammarClass(grammar_root, self._grammar_path)

    def when_fetching_requests_of_action(self, action):
        self._result = list(self._grammar.requests_of_action(action))

    def then_result_is(self, expected_result):
        assert expected_result == self._result

    @contextmanager
    def expect_warnings(self):
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            yield

    def when_fetching_questions_of_predicate_then_warns(self, predicate, expected_warning):
        with patch("%s.warnings" % grammar.__name__) as mocked_warnings:
            self.when_fetching_questions_of_predicate(predicate)
            mocked_warnings.warn.assert_called_once_with(expected_warning)

    def when_fetching_requests_of_action_then_warns(self, action, expected_warning):
        with patch("%s.warnings" % grammar.__name__) as mocked_warnings:
            self.when_fetching_requests_of_action(action)
            mocked_warnings.warn.assert_called_once_with(expected_warning)

    @pytest.mark.parametrize(
        "GrammarClass,grammar", [(GrammarForRGL, "tala/model/grammar/test/grammar_for_rgl_example.xml"),
                                 (Grammar, "tala/model/grammar/test/grammar_example.xml")]
    )
    def test_questions_of_predicate(self, GrammarClass, grammar):
        self.given_ddd_name("rasa_test")
        self.given_individuals_in_ontology(
            sort="contact",
            individuals=[
                "contact_john",
                "contact_john_chi",
                "contact_lisa",
                "contact_mary",
                "contact_andy",
                "contact_andy_chi",
            ]
        )
        self.given_grammar_file(grammar)
        self.given_grammar_from_class(GrammarClass)
        self.when_fetching_questions_of_predicate("phone_number_of_contact")
        self.then_result_is([
            Question("phone_number_of_contact", ["tell me a phone number"], []),
            Question("phone_number_of_contact", ["what is ", "'s number"], [
                RequiredSortalEntity("contact"),
            ]),
            Question(
                "phone_number_of_contact", ["tell me ", "'s number"], [
                    RequiredPropositionalEntity("selected_contact_of_phone_number"),
                ]
            ),
        ])

    def when_fetching_questions_of_predicate(self, predicate):
        self._result = list(self._grammar.questions_of_predicate(predicate))

    @pytest.mark.parametrize(
        "GrammarClass,grammar,results", [
            (
                    GrammarForRGL,
                    "tala/model/grammar/test/grammar_for_rgl_example.xml",
                    [["John"],
                     [u"约翰"],
                     ["Lisa"],
                     ["Mary"],
                     ["Andy"],
                     [u"安迪"]]
            ),
            (
                    Grammar,
                    "tala/model/grammar/test/grammar_example.xml",
                    [["John", "Johnny"],
                     [u"约翰"],
                     ["Lisa"],
                     ["Mary"],
                     ["Andy"],
                     [u"安迪"]]
            )
        ])  # yapf: disable
    def test_entries_of_individual(self, GrammarClass, grammar, results):
        self.given_ddd_name("rasa_test")
        self.given_individuals_in_ontology(
            sort="contact",
            individuals=[
                "contact_john",
                "contact_john_chi",
                "contact_lisa",
                "contact_mary",
                "contact_andy",
                "contact_andy_chi",
            ]
        )
        self.given_grammar_file(grammar)
        self.given_grammar_from_class(GrammarClass)
        self.when_fetching_entries_of_individuals([
            "contact_john",
            "contact_john_chi",
            "contact_lisa",
            "contact_mary",
            "contact_andy",
            "contact_andy_chi",
        ])
        self.then_result_is(results)

    def when_fetching_entries_of_individuals(self, individuals):
        self._result = [self._grammar.entries_of_individual(individual) for individual in individuals]

    @pytest.mark.parametrize(
        "GrammarClass,grammar", [
            (GrammarForRGL, "tala/model/grammar/test/grammar_example_for_rgl_with_single_entries.xml"),
            (Grammar, "tala/model/grammar/test/grammar_example_single_entries.xml"),
        ]
    )
    def test_single_question_entry(self, GrammarClass, grammar):
        self.given_ddd_name("rasa_test")
        self.given_grammar_file(grammar)
        self.given_grammar_from_class(GrammarClass)
        self.when_fetching_questions_of_predicate("phone_number_of_contact")
        self.then_result_is([
            Question("phone_number_of_contact", ["tell me a phone number"], []),
        ])

    @pytest.mark.parametrize(
        "GrammarClass,grammar", [
            (GrammarForRGL, "tala/model/grammar/test/grammar_example_for_rgl_with_single_entries.xml"),
            (Grammar, "tala/model/grammar/test/grammar_example_single_entries.xml"),
        ]
    )
    def test_single_action_entry(self, GrammarClass, grammar):
        self.given_ddd_name("rasa_test")
        self.given_grammar_file(grammar)
        self.given_grammar_from_class(GrammarClass)
        with self.expect_warnings():
            self.when_fetching_requests_of_action("top")
        self.then_result_is([
            Request("top", ["never mind"], []),
        ])

    def test_action_without_plain_text_entries_warns(self):
        self.given_ddd_name("rasa_test")
        self.given_grammar_file("tala/model/grammar/test/grammar_example_without_plain_text_entries.xml")
        self.given_grammar_from_class(Grammar)
        self.when_fetching_requests_of_action_then_warns(
            "call",
            "Grammar ignores element 'action' with attributes {'name': 'call'} since there are no plain text items"
        )

    def test_request_in_rgl_without_plain_text_entries_warns(self):
        self.given_ddd_name("rasa_test")
        self.given_grammar_file("tala/model/grammar/test/grammar_example_for_rgl_without_plain_text_entries.xml")
        self.given_grammar_from_class(GrammarForRGL)
        self.when_fetching_requests_of_action_then_warns(
            "call",
            "GrammarForRGL ignores element 'request' with attributes {'action': 'call'} since it has no <utterance>"
        )

    def test_question_without_plain_text_entries_warns(self):
        self.given_ddd_name("rasa_test")
        self.given_grammar_file("tala/model/grammar/test/grammar_example_without_plain_text_entries.xml")
        self.given_grammar_from_class(Grammar)
        self.when_fetching_questions_of_predicate_then_warns(
            "phone_number_of_contact", "Grammar ignores element 'question' with attributes "
            "{'predicate': 'phone_number_of_contact', 'speaker': 'user'} since there are no plain text items"
        )

    def test_question_in_rgl_without_plain_text_entries_warns(self):
        self.given_ddd_name("rasa_test")
        self.given_grammar_file("tala/model/grammar/test/grammar_example_for_rgl_without_plain_text_entries.xml")
        self.given_grammar_from_class(GrammarForRGL)
        self.when_fetching_questions_of_predicate_then_warns(
            "phone_number_of_contact", "GrammarForRGL ignores element 'question' with attributes "
            "{'predicate': 'phone_number_of_contact', 'speaker': 'user'} since it has no <utterance>"
        )

    @pytest.mark.parametrize(
        "GrammarClass,grammar", [
            (GrammarForRGL, "tala/model/grammar/test/grammar_example_for_rgl_with_answer_entries.xml"),
            (Grammar, "tala/model/grammar/test/grammar_example_with_answer_entries.xml"),
        ]
    )
    def test_answers(self, GrammarClass, grammar):
        self.given_ddd_name("rasa_test")
        self.given_grammar_file(grammar)
        self.given_grammar_from_class(GrammarClass)
        self.when_fetching_user_answers()
        self.then_result_is([
            Answer(["in ", " hours ", " minutes"], [
                RequiredPropositionalEntity("hours"),
                RequiredPropositionalEntity("minutes"),
            ]),
            Answer(["", " hours"], [
                RequiredPropositionalEntity("hours"),
            ]),
            Answer(["in ", " minutes"], [
                RequiredPropositionalEntity("minutes"),
            ]),
            Answer(["", " minutes"], [
                RequiredPropositionalEntity("minutes"),
            ]),
            Answer(["in ", ""], [
                RequiredPropositionalEntity("minutes"),
            ]),
        ])

    def when_fetching_user_answers(self):
        self._result = list(self._grammar.answers())

    @pytest.mark.parametrize(
        "GrammarClass,grammar", [
            (GrammarForRGL, "tala/model/grammar/test/grammar_example_for_rgl_with_answer_entry_without_predicate.xml"),
            (Grammar, "tala/model/grammar/test/grammar_example_with_answer_entry_without_predicate.xml"),
        ]
    )
    def test_answer_entry_without_propositional_fails_fast(self, GrammarClass, grammar):
        self.given_ddd_name("rasa_test")
        self.given_grammar_file(grammar)
        self.given_grammar_from_class(GrammarClass)
        self.when_fetching_user_answers_then_exception_is_raised_matching(
            UnexpectedAnswerFormatException, "Expected at least one <[\w]+ .../> in every item in "
            "<answer speaker=\"user\"> but found some without"
        )

    def when_fetching_user_answers_then_exception_is_raised_matching(self, expected_exception, expected_pattern):
        with pytest.raises(expected_exception, match=expected_pattern):
            self.when_fetching_user_answers()

    @pytest.mark.parametrize(
        "GrammarClass,grammar", [
            (GrammarForRGL, "tala/model/grammar/test/grammar_example_for_rgl_with_string_entries.xml"),
            (Grammar, "tala/model/grammar/test/grammar_example_with_string_entries.xml"),
        ]
    )
    def test_string_entries(self, GrammarClass, grammar):
        self.given_ddd_name("rasa_test")
        self.given_grammar_file(grammar)
        self.given_grammar_from_class(GrammarClass)
        self.when_fetching_strings_of_predicate("comment")
        self.then_result_is([
            "i like movies",
            "i hate movies",
        ])

    def when_fetching_strings_of_predicate(self, predicate):
        with self.expect_warnings():
            self._result = list(self._get_strings_of_predicate(predicate))

    def _get_strings_of_predicate(self, predicate):
        return self._grammar.strings_of_predicate(predicate)

    @pytest.mark.parametrize(
        "GrammarClass,grammar", [
            (GrammarForRGL, "tala/model/grammar/test/grammar_example_for_rgl_without_string_entries.xml"),
            (Grammar, "tala/model/grammar/test/grammar_example_without_string_entries.xml"),
        ]
    )
    def test_no_string_entries(self, GrammarClass, grammar):
        self.given_ddd_name("rasa_test")
        self.given_grammar_file(grammar)
        self.given_grammar_from_class(GrammarClass)
        self.when_fetching_strings_of_predicate("comment")
        self.then_result_is([])

    @pytest.mark.parametrize(
        "GrammarClass,grammar", [
            (GrammarForRGL, "tala/model/grammar/test/grammar_example_for_rgl_without_string_entries.xml"),
            (Grammar, "tala/model/grammar/test/grammar_example_without_string_entries.xml"),
        ]
    )
    def test_warnings_with_no_string_entries(self, GrammarClass, grammar):
        self.given_ddd_name("rasa_test")
        self.given_grammar_file(grammar)
        self.given_grammar_from_class(GrammarClass)
        self.when_fetching_strings_of_predicate_then_warning_is_issued_matching(
            "comment",
            """Expected training examples for predicate 'comment' of sort 'string' but found none. Add them with:

<string predicate="comment">
  <one-of>
    <item>an example</item>
    <item>another example</item>
  </one-of>
</string>"""
        )

    def when_fetching_strings_of_predicate_then_warning_is_issued_matching(self, predicate, expected_warning):
        with patch("%s.warnings" % grammar.__name__) as mocked_warnings:
            self.when_fetching_strings_of_predicate(predicate)
            mocked_warnings.warn.assert_called_once_with(expected_warning)



import os
import os.path
import shutil
import tempfile

from mock import Mock, ANY, patch
import pytest

from tala.ddd.building.steps.generate import GenerateStepForHandcraftedGFFiles, GenerateStepForGFRGLGeneration, GenerateStepForGFGeneration
from tala.ddd.building.steps.verify import VerifyStepForGFGeneration, VerifyStepForGFRGLGeneration, VerifyStepForHandcraftedGFFiles
from tala.ddd.building.steps import step_factory
from tala.ddd.building.steps.step_factory import GenerateStepFactory, VerifyStepFactory
from tala.model.ddd import DDD


class TestStepFactories(object):
    def setup(self):
        self._temp_dir = tempfile.mkdtemp(prefix="DddBuilderTestCase")
        self._cwd = os.getcwd()
        os.chdir(self._temp_dir)
        self._ddd_name = None
        self._mock_ddd = None
        self._language_codes = []
        self._actual_step = None
        self._step_factory = None

    def teardown(self):
        os.chdir(self._cwd)
        shutil.rmtree(self._temp_dir)

    @patch("{}.utils".format(step_factory.__name__), autospec=True)
    @patch("{}.VerifyStepForGFGeneration".format(step_factory.__name__), autospec=True)
    def test_created_verification_step_is_returned(self, MockVerificationStepForGFGeneration, mocked_utils):
        self._given_ddd_name("app")
        self._given_mocked_ddd(use_rgl=False)
        self._given_file_exists("app/grammar/grammar_eng.py")
        self._given_language_codes(["eng"])
        self._given_no_handcrafted_gf_files(mocked_utils)
        self._given_step_factory_created(VerifyStepFactory)
        self._when_creating_step()
        self._then_step_is_returned(MockVerificationStepForGFGeneration)

    def _given_mocked_ddd(self, use_rgl=False):
        mock_ddd = Mock(spec=DDD)
        mock_ddd.name = self._ddd_name
        mock_ddd.use_rgl = use_rgl
        self._mock_ddd = mock_ddd

    def _given_ddd_name(self, name):
        self._ddd_name = name

    def _given_file_exists(self, path):
        self._ensure_dir_exists(os.path.dirname(path))
        with open(path, "w"):
            pass

    def _ensure_dir_exists(self, path):
        if not os.path.exists(path):
            os.makedirs(path)

    def _given_language_codes(self, language_codes):
        self._language_codes = language_codes

    def _given_no_handcrafted_gf_files(self, mocked_utils):
        mocked_utils.has_handcrafted_gf_grammar.return_value = False

    def _given_step_factory_created(self, class_):
        self._step_factory = class_(self._mock_ddd, self._language_codes, verbose=False, ignore_warnings=False)

    def _when_creating_step(self):
        self._actual_step = self._step_factory.create()

    def _then_step_is_returned(self, expected_step):
        assert self._actual_step == expected_step.return_value

    @patch("{}.utils".format(step_factory.__name__), autospec=True)
    @patch("{}.GenerateStepForGFGeneration".format(step_factory.__name__), autospec=True)
    def test_created_generation_step_is_returned(self, MockGenerationStepForGFGeneration, mocked_utils):
        self._given_ddd_name("app")
        self._given_mocked_ddd(use_rgl=False)
        self._given_file_exists("app/grammar/grammar_eng.py")
        self._given_language_codes(["eng"])
        self._given_no_handcrafted_gf_files(mocked_utils)
        self._given_step_factory_created(GenerateStepFactory)
        self._when_creating_step()
        self._then_step_is_returned(MockGenerationStepForGFGeneration)

    @pytest.mark.parametrize(
        "StepFactory,ExpectedStep", [
            (VerifyStepFactory, VerifyStepForGFGeneration),
            (GenerateStepFactory, GenerateStepForGFGeneration),
        ]
    )
    @patch("{}.utils".format(step_factory.__name__), autospec=True)
    def test_creating_step_for_gf_generation(self, mocked_utils, StepFactory, ExpectedStep):
        self._given_ddd_name("app")
        self._given_mocked_ddd(use_rgl=False)
        self._given_file_exists("app/grammar/grammar_eng.py")
        self._given_language_codes(["eng"])
        self._given_no_handcrafted_gf_files(mocked_utils)
        with self._given_expected_step(ExpectedStep) as MockExpectedStep:
            self._given_step_factory_created(StepFactory)
            self._when_creating_step()
            self._then_step_is_created_with_expected_parameters(MockExpectedStep)

    def _given_expected_step(self, ExpectedStep):
        return patch("{}.{}".format(step_factory.__name__, ExpectedStep.__name__), autospec=True)

    def _then_step_is_created_with_expected_parameters(self, MockExpectedStep):
        MockExpectedStep.assert_called_once_with(self._mock_ddd, False, self._language_codes, False, ANY, "grammar")

    @pytest.mark.parametrize(
        "StepFactory,ExpectedStep", [
            (VerifyStepFactory, VerifyStepForGFRGLGeneration),
            (GenerateStepFactory, GenerateStepForGFRGLGeneration),
        ]
    )
    @patch("{}.utils".format(step_factory.__name__), autospec=True)
    def test_verify_creates_step_for_gf_rgl_generation(self, mocked_utils, StepFactory, ExpectedStep):
        self._given_ddd_name("app")
        self._given_mocked_ddd(use_rgl=True)
        self._given_file_exists("app/grammar/grammar_eng.py")
        self._given_language_codes(["eng"])
        self._given_no_handcrafted_gf_files(mocked_utils)
        with self._given_expected_step(ExpectedStep) as MockExpectedStep:
            self._given_step_factory_created(StepFactory)
            self._when_creating_step()
            self._then_step_is_created_with_expected_parameters(MockExpectedStep)

    @pytest.mark.parametrize(
        "StepFactory,ExpectedStep", [
            (VerifyStepFactory, VerifyStepForHandcraftedGFFiles),
            (GenerateStepFactory, GenerateStepForHandcraftedGFFiles),
        ]
    )
    @patch("{}.utils".format(step_factory.__name__), autospec=True)
    def test_verify_creates_step_for_handcrafted_grammar(self, mocked_utils, StepFactory, ExpectedStep):
        self._given_ddd_name("app")
        self._given_mocked_ddd(use_rgl=False)
        self._given_file_exists("app/grammar/grammar_eng.py")
        self._given_language_codes(["eng"])
        self._given_handcrafted_gf_files(mocked_utils)
        with self._given_expected_step(ExpectedStep) as MockExpectedStep:
            self._given_step_factory_created(StepFactory)
            self._when_creating_step()
            self._then_step_is_created_with_expected_parameters(MockExpectedStep)

    def _given_handcrafted_gf_files(self, mocked_utils):
        mocked_utils.has_handcrafted_gf_grammar.return_value = True



import unittest

from tala.model.date_time import DateTime


class DateTimeTestCase(unittest.TestCase):
    def test_unicode(self):
        self.when_get_unicode_string(DateTime("2018-04-11T22:00:00.000Z"))
        self.then_result_is('datetime(2018-04-11T22:00:00.000Z)')

    def when_get_unicode_string(self, date_time):
        self._actual_result = unicode(date_time)

    def then_result_is(self, expected_result):
        self.assertEquals(expected_result, self._actual_result)

    def test_human_standard(self):
        self.when_get_human_standard(DateTime("2018-04-11T22:00:00.000Z"))
        self.then_result_is("04/11/2018 10:00 PM")

    def when_get_human_standard(self, date_time):
        self._actual_result = date_time.human_standard()

    def test_repr(self):
        self.when_get_repr(DateTime("2018-04-11T22:00:00.000Z"))
        self.then_result_is('DateTime("2018-04-11T22:00:00.000Z")')

    def when_get_repr(self, date_time):
        self._actual_result = repr(date_time)



#!/usr/bin/env python2.7

import StringIO
import codecs
import os
import sys
from collections import OrderedDict

from lxml import etree

from tala.nl import languages
from tala.config import BackendConfig
from tala.ddd.ddd_py_compiler import DddPyCompiler
from tala.model.plan_item import PlanItem
from tala.nl.gf import rgl_grammar_entry_types as rgl_types
from tala.nl.gf.grammar_entry_types import Constants, Node


class PyToXmlConverter:
    def __init__(self, ddd, languages):
        self._ddd = ddd
        self._languages = languages

    def convert(self):
        OntologyConverter(self._ddd, self._languages).convert()
        DomainConverter(self._ddd, self._languages).convert()
        GrammarConverter(self._ddd, self._languages).convert()

    def _create_element(self, tag_name, **attributes):
        element = etree.Element(tag_name, OrderedDict())
        self._set_attributes(element, **attributes)
        return element

    def _set_attributes(self, element, **attributes):
        for attribute, value in attributes.iteritems():
            if value is not None:
                self._set_attribute(element, attribute, unicode(value))

    def _set_attribute(self, element, name, value):
        element.attrib[name] = value

    def _write_xml_header(self):
        self._write('<?xml version="1.0" encoding="utf-8"?>\n')

    def _write(self, string):
        self._output.write(string)


class OntologyConverter(PyToXmlConverter):
    def convert(self):
        self._root = self._create_element("ontology", name=self._ddd.ontology.get_name())
        self._convert_sorts()
        self._convert_predicates()
        self._convert_actions()
        self._convert_individuals()
        xml = etree.tostring(self._root, pretty_print=True)
        with open("%s/ontology.xml" % self._ddd.name, "w") as f:
            f.write(xml)

    def _convert_sorts(self):
        for sort in self._ddd.ontology.get_sorts().values():
            if not sort.is_builtin():
                self._convert_sort(sort)

    def _convert_sort(self, sort):
        element = self._create_element("sort", name=sort.get_name())
        if sort.is_dynamic():
            self._set_attribute(element, "dynamic", "true")
        self._root.append(element)

    def _convert_predicates(self):
        for predicate in self._ddd.ontology.get_predicates().values():
            self._convert_predicate(predicate)

    def _convert_predicate(self, predicate):
        element = self._create_element("predicate", name=predicate.get_name())
        self._set_attribute(element, "sort", predicate.getSort().get_name())
        if predicate.allows_multiple_instances():
            self._set_attributes(element, multiple_instances="true")
        if predicate.get_feature_of_name():
            self._set_attributes(element, feature_of=predicate.get_feature_of_name())
        self._root.append(element)

    def _convert_actions(self):
        for action in self._ddd.ontology.get_actions():
            if action != "up":
                self._convert_action(action)

    def _convert_action(self, action):
        element = self._create_element("action", name=action)
        self._root.append(element)

    def _convert_individuals(self):
        individuals = self._ddd.ontology.get_individuals()
        for individual, sort in individuals.iteritems():
            self._convert_individual(individual, sort.get_name())

    def _convert_individual(self, individual, sort_name):
        element = self._create_element("individual", name=individual)
        self._set_attribute(element, "sort", sort_name)
        self._root.append(element)


class DomainConverter(PyToXmlConverter):
    def convert(self):
        self._root = self._create_element("domain", name=self._ddd.domain.get_name())
        self._convert_goals()
        self._convert_dependencies()
        self._convert_parameters()
        self._convert_default_questions()
        xml = etree.tostring(self._root, pretty_print=True)
        xml = xml.replace("</goal>", "</goal>\n")
        with open("%s/domain.xml" % self._ddd.name, "w") as f:
            f.write(xml)

    def _convert_goals(self):
        for goal in self._ddd.domain.get_all_goals_in_defined_order():
            if not (goal.is_perform_goal() and goal.get_action().get_value() == "up"):
                self._convert_goal(goal)

    def _convert_goal(self, goal):
        element = self._create_element("goal")
        self._set_goal_attributes(element, goal)
        if self._ddd.domain.has_dynamic_title(goal):
            self._set_attributes(element, dynamic_title="true")
        if self._ddd.domain.goal_allows_accommodation_without_feedback(goal):
            self._set_attributes(element, accommodate_without_feedback="true")
        if self._ddd.domain.restart_on_completion(goal):
            self._set_attributes(element, restart_on_completion="true")
        io_status = self._ddd.domain.get_goal_attribute(goal, "io_status")
        if io_status:
            self._set_attributes(element, io_status=io_status)
        self._add_preferred_element_if_present(goal, element)
        self._convert_plan(element, "plan", self._ddd.domain.get_plan(goal))
        self._convert_postconds(element, self._ddd.domain.get_postconds(goal))
        if self._ddd.domain.get_postplan(goal):
            self._convert_plan(element, "postplan", self._ddd.domain.get_postplan(goal))
        self._convert_gui_context(goal, element)
        self._convert_superactions(goal, element)
        self._root.append(element)

    def _set_goal_attributes(self, element, goal):
        if goal.is_perform_goal():
            self._set_attribute(element, "type", "perform")
            self._set_attribute(element, "action", goal.get_action().get_value())
        elif goal.is_resolve_goal():
            question = goal.get_question()
            self._set_attribute(element, "type", "resolve")
            self._set_question_attributes(element, question, "question_type")
        elif goal.is_handle_goal():
            self._set_attribute(element, "type", "handle")
            self._set_attribute(element, "action", goal.get_device_event())
        else:
            raise Exception("unsupported goal %s" % goal)
        if self._ddd.domain.goal_allows_accommodation_without_feedback(goal):
            self._set_attribute(element, "accommodate_without_feedback", "true")

    def _add_preferred_element_if_present(self, goal, goal_element):
        node = self._ddd.domain.get_goal_attribute(goal, "preferred")
        if node:
            preferred_element = self._create_element("preferred")
            goal_element.append(preferred_element)
            if node is not True and node.is_predicate_proposition():
                proposition_element = self._create_element(
                    "proposition", predicate=node.getPredicate(), value=node.getArgument()
                )
                preferred_element.append(proposition_element)

    def _create_goal_element(self, goal):
        if goal.is_perform_goal():
            type_ = "perform"
        if goal.is_resolve_goal():
            type_ = "resolve"
        if goal.is_handle_goal():
            type_ = "handle"
        element = self._create_element(type_)
        if goal.is_perform_goal():
            self._set_attribute(element, "action", goal.get_action().get_value())
        elif goal.is_resolve_goal():
            question = goal.get_question()
            self._set_question_attributes(element, question, "type")
        elif goal.is_handle_goal():
            self._set_attribute(element, "action", goal.get_device_event())
        else:
            raise Exception("unsupported goal %s" % goal)
        return element

    def _set_question_attributes(self, element, question, type_attribute="type"):
        if question.is_wh_question():
            if question.get_content().is_lambda_abstracted_goal_proposition():
                self._set_attribute(element, type_attribute, "goal")
            else:
                self._set_attribute(element, type_attribute, "wh_question")
                if question.get_content().is_lambda_abstracted_predicate_proposition():
                    self._set_attributes(element, predicate=question.get_predicate())
        elif question.is_alt_question():
            self._set_attribute(element, type_attribute, "alt_question")
        elif question.is_yes_no_question():
            self._set_attribute(element, type_attribute, "yn_question")
            if question.get_content().is_goal_proposition():
                if question.get_content().get_goal().is_perform_goal():
                    action = question.get_content().get_goal().get_content()
                    perform_element = self._create_element("perform", action=action)
                    element.append(perform_element)
                elif question.get_content().get_goal().is_resolve_goal():
                    predicate = question.get_content().get_goal().get_content().get_predicate()
                    perform_element = self._create_element("resolve", type="wh_question", predicate=predicate)
                    element.append(perform_element)
            else:
                proposition_element = self._create_element("proposition")
                self._set_proposition_attributes(proposition_element, question.get_content())
                element.append(proposition_element)
        else:
            raise Exception("unsupported question %s" % question)

    def _set_proposition_attributes(self, element, proposition, typed_by_parent=False):
        if proposition.is_goal_proposition():
            self._set_goal_attributes(element, proposition.get_goal())
        elif proposition.is_predicate_proposition():
            self._set_predicate_proposition_attributes(element, proposition, typed_by_parent)
        else:
            raise Exception("unsupported proposition %s" % proposition)

    def _set_predicate_proposition_attributes(self, element, proposition, typed_by_parent=False):
        if not typed_by_parent:
            self._set_attributes(element, type="predicate")
            self._set_attributes(element, predicate=proposition.getPredicate())
        if proposition.getArgument() is not None:
            self._set_attributes(element, value=proposition.getArgument().getValue())

    def _convert_plan(self, parent, tag, plan):
        element = self._create_element(tag)
        for item in plan:
            self._convert_plan_item(element, item)
        parent.append(element)

    def _convert_plan_item(self, parent, item):
        if item.get_type() in [PlanItem.TYPE_FINDOUT, PlanItem.TYPE_RAISE, PlanItem.TYPE_BIND]:
            self._convert_question_raising_item(parent, item)
        # elif item.get_type() == PlanItem.TYPE_IF_THEN_ELSE:
        #     self._convert_if_then_else(parent, item)
        elif item.get_type() == PlanItem.TYPE_FORGET:
            self._convert_forget(parent, item)
        elif item.get_type() == PlanItem.TYPE_FORGET_ALL:
            self._convert_forget_all(parent)
        elif item.get_type() == PlanItem.TYPE_INVOKE_SERVICE_QUERY:
            self._convert_invoke_service_query(parent, item)
        elif item.get_type() == PlanItem.TYPE_INVOKE_SERVICE_ACTION:
            self._convert_invoke_service_action(parent, item)
        elif item.get_type() == PlanItem.TYPE_JUMPTO:
            self._convert_jumpto(parent, item)
        elif item.get_type() == PlanItem.TYPE_ASSUME_SHARED:
            self._convert_assume_shared(parent, item)
        else:
            raise Exception("unsupported plan item %s" % item)

    def _convert_question_raising_item(self, parent, item):
        element = self._create_element(item.get_type())
        self._set_question_attributes(element, item.get_question())
        self._add_potential_question_alts(element, item.get_question())
        parent.append(element)

    def _add_potential_question_alts(self, parent, question):
        if question.is_alt_question():
            self._add_alts(parent, question.get_content())

    def _add_alts(self, parent, alts):
        for alt in alts:
            element = self._create_alt_element(alt)
            parent.append(element)

    def _create_alt_element(self, alt):
        element = self._create_element("alt")
        if alt.is_goal_proposition():
            child = self._create_goal_element(alt.get_goal())
            element.append(child)
        elif alt.is_predicate_proposition():
            child = self._create_element("proposition")
            self._set_proposition_attributes(child, alt)
            element.append(child)
        else:
            raise Exception("unsupported alt %s" % alt)
        return element

    def _convert_forget(self, parent, item):
        element = self._create_element("forget")
        if item.getContent().is_predicate_proposition():
            self._set_predicate_proposition_attributes(element, item.getContent())
        elif item.getContent().is_predicate():
            self._set_attribute(element, "predicate", item.getContent().get_name())
        parent.append(element)

    def _convert_forget_all(self, parent):
        element = self._create_element("forget_all")
        parent.append(element)

    def _convert_invoke_service_query(self, parent, item):
        element = self._create_element("invoke_service_query")
        self._set_question_attributes(element, item.getContent())
        parent.append(element)

    def _convert_invoke_service_action(self, parent, item):
        element = self._create_element("invoke_service_action", action=item.get_service_action())
        if item.preconfirm:
            self._set_attribute(element, "preconfirm", item.preconfirm.lower())
        if item.postconfirm:
            self._set_attribute(element, "postconfirm", "true")
        if item.should_downdate_plan():
            self._set_attribute(element, "downdate_plan", "true")
        parent.append(element)

    def _convert_postconds(self, parent, postconds):
        for postcond in postconds:
            self._convert_postcond(parent, postcond)

    def _convert_postcond(self, parent, postcond):
        element = self._create_element("postcond")
        if postcond.is_predicate_proposition():
            predicate = postcond.getPredicate()
            individual = postcond.getArgument()
            child = self._create_element("proposition", predicate=predicate, value=individual)
            element.append(child)
        parent.append(element)

    def _convert_jumpto(self, parent, item):
        element = self._create_element("jumpto")
        self._set_goal_attributes(element, item.getContent())
        parent.append(element)

    def _convert_assume_shared(self, parent, item):
        element = self._create_element("assume_shared")
        proposition_element = self._create_element("proposition")
        self._set_proposition_attributes(proposition_element, item.getContent())
        element.append(proposition_element)
        parent.append(element)

    def _convert_dependencies(self):
        for dependent_question, others in self._ddd.domain.dependencies.iteritems():
            element = self._create_element("dependency")
            self._set_question_attributes(element, dependent_question)
            for other in others:
                child = self._create_element("question")
                self._set_question_attributes(child, other)
                element.append(child)
            self._root.append(element)

    def _convert_parameters(self):
        for obj, parameters in self._ddd.domain.parameters.iteritems():
            self._convert_object_parameters(obj, parameters)

    def _convert_object_parameters(self, obj, parameters):
        element = self._create_element("parameters")
        if obj.is_question():
            self._set_question_attributes(element, obj, "question_type")
        elif obj.is_predicate():
            self._set_attribute(element, "predicate", obj.get_name())
        else:
            raise Exception("parameters supported only for questions and predicates")
        for name, value in parameters.iteritems():
            self._convert_parameter(element, name, value)
        self._root.append(element)

    def _convert_parameter(self, parent, parameter, value):
        if parameter == "alts":
            self._add_alts(parent, value)
        elif parameter == "background":
            self._add_background_parameter(parent, value)
        elif parameter in ["service_query", "default"]:
            self._set_question_valued_parameter(parent, parameter, value)
        elif parameter == "related_information":
            self._set_questions_valued_parameter(parent, parameter, value)
        elif parameter == "label_questions":
            self._set_questions_valued_parameter(parent, "label_question", value)
        elif parameter in ["incremental", "verbalize"]:
            self._set_attribute(parent, parameter, self._convert_boolean_value(value))
        elif parameter == "ask_features":
            self._add_ask_features(parent, value)
        elif parameter in [
            "graphical_type", "source", "format", "sort_order", "allow_goal_accommodation", "max_spoken_alts"
        ]:
            self._set_attribute(parent, parameter, unicode(value))
        else:
            raise Exception("unsupported parameter %r" % parameter)

    def _convert_boolean_value(self, value):
        if value:
            return "true"
        else:
            return "false"

    def _add_background_parameter(self, parent, predicates):
        for predicate in predicates:
            element = self._create_element("background", predicate=predicate)
            parent.append(element)

    def _set_question_valued_parameter(self, parent, parameter, value):
        element = self._create_element(parameter)
        self._set_question_attributes(element, value)
        parent.append(element)

    def _set_questions_valued_parameter(self, parent, parameter, questions):
        for question in questions:
            element = self._create_element(parameter)
            self._set_question_attributes(element, question)
            parent.append(element)

    def _convert_gui_context(self, goal, goal_element):
        gui_context = self._ddd.domain.get_gui_context(goal)
        for item in gui_context:
            predicate = item.get_name()
            element = self._create_element("gui_context", predicate=predicate)
            goal_element.append(element)

    def _convert_superactions(self, goal, goal_element):
        for superaction in self._ddd.domain.get_superactions(goal):
            superaction_element = self._create_element("superaction", name=superaction.get_value())
            goal_element.append(superaction_element)

    def _add_ask_features(self, parent, predicates):
        for predicate in predicates:
            ask_feature_element = self._create_element("ask_feature", predicate=predicate)
            parent.append(ask_feature_element)

    def _convert_default_questions(self):
        for question in self._ddd.domain.default_questions:
            if question.is_wh_question():
                predicate = question.get_predicate()
                question_element = self._create_element("default_question", type="wh_question", predicate=predicate)
                self._root.append(question_element)


class GrammarConverter(PyToXmlConverter):
    def __init__(self, ddd, languages):
        PyToXmlConverter.__init__(self, ddd, languages)
        self._conversion_methods = {
            Constants.GRAMMAR: self._convert_grammar_node,
            Constants.ITEM: self._convert_item_node,
            Constants.ONE_OF: self._convert_one_of_node,
            Constants.PREDICATE: self._convert_predicate_node,
            Constants.VALIDITY: self._convert_validity_node,
            Constants.ACTION: self._convert_action_node,
            Constants.PREREPORT: self._convert_prereport_node,
            Constants.REPORT_STARTED: self._convert_report_started_node,
            Constants.REPORT_ENDED: self._convert_report_ended_node,
            Constants.REPORT_FAILED: self._convert_report_failed_node,
            Constants.PRECONFIRM: self._convert_preconfirm_node,
            Constants.SLOT: self._convert_slot_node,
            Constants.NP: self._convert_np_node,
            Constants.ACTION_TITLE: self._convert_action_title_node,
            Constants.ISSUE_TITLE: self._convert_issue_title_node,
            Constants.SYS_QUESTION: self._convert_sys_question_node,
            Constants.USER_QUESTION: self._convert_user_question_node,
            Constants.INDIVIDUAL: self._convert_individual,
            Constants.ANSWER_COMBINATION: self._convert_answer_combination,
            Constants.VP: self._convert_simple_node,
            Constants.INDEFINITE: self._convert_simple_node,
            Constants.DEFINITE: self._convert_simple_node,
            Constants.SYS_ANSWER: self._convert_sys_answer,
            Constants.INFINITIVE: self._convert_simple_node,
            Constants.IMPERATIVE: self._convert_simple_node,
            Constants.ING_FORM: self._convert_simple_node,
            Constants.OBJECT: self._convert_simple_node,
            Constants.GREETING: self._convert_simple_node,
            Constants.POSITIVE_SYS_ANSWER: self._convert_positive_sys_answer,
            Constants.NEGATIVE_SYS_ANSWER: self._convert_negative_sys_answer,
            rgl_types.UTTERANCE: self._convert_utterance,
        }

    def convert(self):
        for language_code in self._languages:
            grammar = self._load_and_compile_grammar_entries(language_code)
            self._convert_grammar(language_code, grammar)

    def _load_and_compile_grammar_entries(self, language_code):
        grammar_source = self._load_grammar_source(language_code)
        return DddPyCompiler().compile_grammar(grammar_source, self._ddd.ontology, self._ddd.service_interface)

    def _load_grammar_source(self, language_code):
        with open("%s/grammar/grammar_%s.py" % (self._ddd.name, language_code)) as f:
            return f.read()

    def _convert_grammar(self, language_code, input_root_node):
        self._output = codecs.open("%s/grammar/grammar_%s.xml" % (self._ddd.name, language_code), "w", "utf-8")
        self._generate_and_write_output(input_root_node)
        self._output.close()

    def _generate_and_write_output(self, input_root_node):
        self._write_xml_header()
        self._convert_node(input_root_node)

    def _convert_node(self, node, parent=None):
        if isinstance(node, (str, unicode)):
            if len(node) > 0:
                self._write(node)
            else:
                print "warning: can not convert empty grammar node: %s" % parent
        elif node.type in self._conversion_methods:
            self._conversion_methods[node.type](node, parent)
        else:
            print "WARNING: don't know how to convert grammar node of type %r" % node.type

    def _convert_grammar_node(self, node, parent):
        self._write('<grammar>\n')
        self._convert_children(node.children, parent, delimeter="\n")
        self._write('</grammar>\n')

    def _convert_children(self, children, parent, itemize=False, indent=0, delimeter=""):
        for idx in range(len(children)):
            child = children[idx]
            if itemize:
                self._write('\n%s<item>' % ("  " * indent))
            self._convert_node(child, parent)
            if itemize:
                self._write('</item>')
            if idx < (len(children) - 1):
                self._write(delimeter)

    def _convert_simple_node(self, node, parent):
        self._write_element(node, 0, trailing_newline=False)

    def _convert_item_node(self, node, parent):
        self._convert_children(node.children, parent, indent=2)

    def _convert_one_of_node(self, node, parent):
        self._write('\n')
        self._write_element(node, 2, itemize_children=True)
        self._write('  ')

    def _convert_predicate_node(self, node, parent):
        attributes = {"type": "wh_question", "speaker": "all", "predicate": node.parameters["name"]}
        self._write_element(node, 1, attributes, "question")

    def _convert_validity_node(self, node, parent):
        attributes = {"name": node.parameters["name"]}
        self._write_element(node, 1, attributes)

    def _convert_action_node(self, node, parent):
        attributes = {"name": node.parameters["name"]}
        self._write_element(node, 1, attributes)

    def _convert_prereport_node(self, node, parent):
        attributes = {"action": node.parameters["action"], "status": "started", "source": "dialogue"}
        self._write_element(node, 1, attributes, "report")

    def _convert_report_started_node(self, node, parent):
        attributes = {"action": node.parameters["action"], "status": "started"}
        self._write_element(node, 1, attributes, "report")

    def _convert_report_ended_node(self, node, parent):
        attributes = {"action": node.parameters["action"], "status": "ended"}
        self._write_element(node, 1, attributes, "report")

    def _convert_report_failed_node(self, node, parent):
        attributes = {"action": node.parameters["action"], "status": "failed", "reason": node.parameters["reason"]}
        self._write_element(node, 1, attributes, "report")

    def _convert_preconfirm_node(self, node, parent):
        attributes = {"action": node.parameters["action"]}
        self._write_element(node, 1, attributes)

    def _convert_slot_node(self, node, parent):
        if "predicate" in node.parameters:
            attributes = {"type": "individual", "predicate": node.parameters["predicate"]}
            self._write_element(node, 0, attributes, trailing_newline=False)
        elif "predicate" in parent.parameters:
            attributes = {"type": "individual", "predicate": parent.parameters["predicate"]}
            self._write_element(node, 0, attributes, trailing_newline=False)
        elif "sort" in node.parameters:
            attributes = {"sort": node.parameters["sort"]}
            self._write_element(node, 0, attributes, trailing_newline=False)

    def _convert_np_node(self, node, parent):
        self._write('\n')
        self._write_element(node, 2)

    def _convert_action_title_node(self, node, parent):
        attributes = {"type": "action", "action": node.parameters["action"]}
        self._write_element(node, 1, attributes, "title")

    def _convert_issue_title_node(self, node, parent):
        attributes = {"type": "question", "predicate": node.parameters["predicate"]}
        self._write_element(node, 1, attributes, "title")

    def _convert_sys_question_node(self, node, parent):
        attributes = {"type": "wh_question", "predicate": node.parameters["predicate"], "speaker": "system"}
        self._write_element(node, 1, attributes, "question")

    def _convert_user_question_node(self, node, parent):
        attributes = {"type": "wh_question", "predicate": node.parameters["predicate"], "speaker": "user"}
        self._write_element(node, 1, attributes, "question")

    def _convert_individual(self, node, parent):
        attributes = {"name": node.parameters["name"]}
        self._write_element(node, 1, attributes)

    def _convert_answer_combination(self, node, parent):
        attributes = {"speaker": "user"}
        self._write_element(node, 1, attributes, "answer")

    def _convert_sys_answer(self, node, parent):
        attributes = self._handle_potential_multi_slot_system_answer(node)
        self._write_element(node, 1, attributes, "answer")

    def _convert_positive_sys_answer(self, node, parent):
        attributes = {"speaker": "system", "predicate": node.parameters["predicate"], "polarity": "positive"}
        self._write_element(node, 1, attributes, "answer")

    def _convert_negative_sys_answer(self, node, parent):
        attributes = {"speaker": "system", "predicate": node.parameters["predicate"], "polarity": "negative"}
        self._write_element(node, 1, attributes, "answer")

    def _write_element(
        self, node, indent=0, attributes={}, override_name=None, itemize_children=False, trailing_newline=True
    ):
        if override_name is not None:
            name = override_name
        else:
            name = node.type
        self._write("%s<%s" % ("  " * indent, name))
        for key, value in attributes.iteritems():
            self._write(' %s="%s"' % (key, value))

        if len(node.children) == 0:
            self._write("/>")
        else:
            self._write(">")
            self._convert_children(node.children, node, itemize=itemize_children, indent=indent + 1)
            if itemize_children:
                self._write("\n%s</%s>" % ("  " * indent, name))
            else:
                self._write("</%s>" % name)

        if trailing_newline:
            self._write("\n")

    def _handle_potential_multi_slot_system_answer(self, node):
        if self._count_answer_slots(node) > 1:
            return {"speaker": "system", "predicate": node.parameters["predicate"]}
        else:
            return {"speaker": "system"}

    def _count_answer_slots(self, node):
        answer_slots = []
        answer_slots = self._collect_answer_slots(node, answer_slots)
        return len(answer_slots)

    def _collect_answer_slots(self, node, answer_slots):
        for child in node.children:
            if child.__class__ == Node:
                if len(child.children) > 0:
                    self._collect_answer_slots(child, answer_slots)
                if child.type == Constants.SLOT:
                    answer_slots.append(child)
        return answer_slots

    def convert_node(self, node):
        self._output = StringIO.StringIO()
        self._convert_node(node)
        return self._output.getvalue()

    def _convert_utterance(self, node, parent):
        self._write('\n')
        self._write_element(node, 2)
        self._write('  ')


if __name__ == "__main__":
    import argparse
    from tala.ddd.loading.ddd_set_loader import DDDSetLoader

    parser = argparse.ArgumentParser()
    parser.add_argument("--ddds", "-d", nargs="+", dest="ddds", required=True)
    parser.add_argument("--languages", nargs="+", choices=languages.SUPPORTED_LANGUAGES, required=True)
    args = parser.parse_args()

    sys.path.insert(0, os.getcwd())
    config = BackendConfig().read()
    ddd_set_loader = DDDSetLoader()
    ddds = ddd_set_loader.ddds_as_list(args.ddds, languages=args.languages)
    for ddd in ddds:
        print "Converting %s to XML" % ddd.name
        PyToXmlConverter(ddd, args.languages).convert()
